#!/usr/bin/python

from __future__ import generators
import sys
from copy import copy
from debug import Debug
from bipartitematch import bipartiteMatch
from time import sleep

SEG_SIZE = 6
SEG_START1 = 0
SEG_START2 = 1
SEG_END1 = 2
SEG_END2 = 3
SEG_IDENT = 4
SEG_SCORE = 5

INF = 1000000000000
SEG_TERMINAL = [INF, INF, INF-1, INF-1, -1, -1]

class Alignment(object):
    '''Alignment is a class representing an alignment in form of list of segments.
    Unless explicitely stated, no position can be covered by more than one match in alignment.
    Segments can cross, though.
    '''
    
    ALLOW_OVERLAP = True
    DONT_ALLOW_OVERLAP = False
    NO_SORT_REASON = 0
    
    
    def __init__(self, seq1, file1, seq1_start, seq1_end, seq2, file2, seq2_start, seq2_end, segments, allow_overlap=False):
        self.seq1 = seq1
        self.file1 = file1
        self.seq1_start = int(seq1_start)
        self.seq1_end = int(seq1_end)
        self.seq2 = seq2
        self.file2 = file2
        self.seq2_start = int(seq2_start)
        self.seq2_end = int(seq2_end)
        self.segments = [ [ int(item) for item in seg ] for seg in segments]
        
        if not allow_overlap:
            
            # remove overlapping alignments
            self.fragmentSegments()
        
            n_old_segs = len(self.segments)
            self.remove_overlap()
            if Debug.debug:
                pass
                #print >>sys.stderr, n_old_segs-len(self.segments), " segments removed because of overlap"
            
            self.assertSingleCoverage()
            
    def _remove_overlap(self, seg_start, seg_end, seg_value = SEG_SCORE):
        removed = 0
        if len(self.segments) > 0:
            new_segments = [ self.segments[0] ]
            for i in xrange(1, len(self.segments)):
                assert self.segments[i-1][seg_start] <= self.segments[i][seg_start]
                seg = self.segments[i]
                lastseg = new_segments[-1]
                if lastseg[seg_end] >= seg[seg_start]:
                    if seg[seg_value] > lastseg[seg_value]:
                        new_segments[-1] = seg
                        removed += 1
                else:
                    new_segments.append(seg)
                    
            self.segments = new_segments
        return removed
        
    def remove_overlap(self):
        '''If there are overlapping alignments, resolve the conflict
        by removing the lower scoring alignment'''
        
        removed = 0;
        
        # the order of the two lines is important
        self.sort(SEG_START2)
        removed += self._remove_overlap(SEG_START2, SEG_END2)
        # leaving the alingnments sorted
        self.sort(SEG_START1)
        removed += self._remove_overlap(SEG_START1, SEG_END1)
        
        return removed
        
    def flip(self):
        raise "Not implemented"
    flip = classmethod(flip)
    
    def coverageCount(self, seg_start, seg_end, seq_len):
        '''Calculate how many times postions are covered with matches'''
        # allocate the memory
        coverage = [ 0 ] * (seq_len+1);
        
        # calculate the coverage of positions
        for seg in self.segments:
            for i in xrange(seg[seg_start], seg[seg_end]+1):
                coverage[i] += 1
                
        return coverage
    
    def coverageCountSeq1(self):
        return self.coverageCount(SEG_START1, SEG_END1, self.seq1_end)
        
    def coverageCountSeq2(self):
        return self.coverageCount(SEG_START2, SEG_END2, self.seq2_end)
        
    def assertSingleCoverage(self):
        '''Asserts no position participates in more than 1 match'''
        c = self.coverageCountSeq1()
        for i in xrange(0, len(c)):
            assert c[i] <= 1, "There is position "+str(i)+" in seq1 "+str(self)+"covered by "+str(c[i])+" segments."
        
        c = self.coverageCountSeq2()
        for i in xrange(0, len(c)):
            assert c[i] <= 1, "There is position "+str(i)+" in seq2 "+str(self)+"covered by "+str(c[i])+" segments."
        
        
    def scoreCoverageCount(self, seg_start, seg_end, seq_start, seq_end):
        '''Calculates sum matches score for each residue.'''
        # allocate the memory
        coverage = [ 0 ] * ((seq_end)+1);
        
        # calculate the score coverage of positions
        for seg in self.segments:
            for i in xrange(seg[seg_start], seg[seg_end]+1):
                coverage[i] += seg[SEG_SCORE]
                
#        # allocate the memory
#        coverage = [ 0 ] * ((seq_end-seq_start)+1);
#        
#        # calculate the score coverage of positions
#        for seg in self.segments:
#            for i in xrange(seg[seg_start], seg[seg_end]+1):
#                coverage[i-seq_start] += seg[SEG_SCORE]
#                
        return coverage
    
    def scoreCoverageCountSeq1(self):
        return self.scoreCoverageCount(SEG_START1, SEG_END1, self.seq1_start, self.seq1_end)
        
    def scoreCoverageCountSeq2(self):
        return self.scoreCoverageCount(SEG_START2, SEG_END2, self.seq2_start, self.seq2_end)

    def segBefore(seg1, seg2, start, end):
        '''Calculate the segment before the intersection. Returns negative size segment if there's nothing before'''
        if seg1[start] <= seg2[start]:
            left = seg1
            right = seg2
        else:
            left = seg2
            right = seg1
        
        len = min(left[end], right[start]-1) - left[start] # real length is len + 1
        
        return [ left[SEG_START1], \
            left[SEG_START2], \
            left[SEG_START1] + len, \
            left[SEG_START2] + len, \
            left[SEG_IDENT], \
            left[SEG_SCORE] ]
            
    segBefore = staticmethod(segBefore)

    def segAfterCommon(seg1, seg2, start, end):
        '''Calculate the segment before the intersection. Returns negative size segment if there's nothing before'''
        if seg1[end] <= seg2[end]:
            left = seg1
            right = seg2
        else:
            left = seg2
            right = seg1
        
        len = right[end] - max(left[end]+1, right[start]) # real length is len + 1
        
        return [ right[SEG_END1] - len, \
            right[SEG_END2] - len, \
            right[SEG_END1], \
            right[SEG_END2], \
            right[SEG_IDENT], \
            right[SEG_SCORE] ]
            
    segAfterCommon = staticmethod(segAfterCommon)

    def segAfterBefore(left, right, start, end):
        '''Calculate the part of the segment seg1 which is left after taking "before" part'''
        assert left[start] <= right[start]
        
        skip = max(left[start], right[start]) - left[start]
        len = left[end] - min(left[end]+1, right[start])
        return [ left[SEG_START1]+skip, \
            left[SEG_START2]+skip, \
            left[SEG_START1]+skip+len, \
            left[SEG_START2]+skip+len, \
            left[SEG_IDENT], \
            left[SEG_SCORE] ]
            
    segAfterBefore = staticmethod(segAfterBefore)

    def genSegments(segments, offset, nextseg, firstseg):
        '''Generates subsequent segments, starting from firstseg(offset),
        next one determined with nextseg, when all segments iterated, ends with?'''
        
        if firstseg.has_key(offset):
            i = firstseg[offset]
            while i != -1:
                yield (segments[i])
                i = nextseg[i]
                
        while True:
            yield (SEG_TERMINAL)
            
    genSegments = staticmethod(genSegments)
            
    def genAllSegments(segments):
        '''Generates subsequent segments'''
        
        for i in xrange(0, len(segments)):
            yield segments[i]
                
        while True:
            yield SEG_TERMINAL
            
    genAllSegments = staticmethod(genAllSegments)
            
    def genSumKeys(dict1, dict2):
        '''Generates keys of sum of the dictionaries'''

        for k in dict1.iterkeys():
            yield(k)

        for k in dict2.iterkeys():
            if not dict1.has_key(k): yield(k)
            
    genSumKeys = staticmethod(genSumKeys)
            
    def createOff2Seg(segments, checkOverlap = Debug.debug):
        '''Return linked list of segments with the same offset 
        + hash of offsets, pointing at the first segment with this offset'''
        nextSeg = [-1] * len(segments)
        off2seg = dict()
        for i in xrange(len(segments)-1, -1, -1):
            seg = segments[i]
            off = seg[SEG_START1] - seg[SEG_START2]
            if off2seg.has_key(off):
                nexti = off2seg[off]
                if checkOverlap: assert segments[i][SEG_END1] < segments[nexti][SEG_START1], 'Assume its sorted and not intersecting '+str(segments[i])+" "+str(segments[i][SEG_START1] - segments[i][SEG_START2])+" "+str(segments[nexti])+" "+str(segments[nexti][SEG_START1] - segments[nexti][SEG_START2])
                nextSeg[i] = nexti
            off2seg[off] = i
            
        return nextSeg, off2seg
                    
    createOff2Seg = staticmethod(createOff2Seg)
    
    def merge_segments(segments1, segments2):
        '''This private method merges the alignments, adding scores where the matches overlap.
        Input: in segment2, no residue is covered by two segments (assures ordering)
        After calling merge method this constraint is not met for self!'''
        
        # sorting the data
        Alignment.sort_segments(segments1, SEG_START1)
        Alignment.sort_segments(segments2, SEG_START1)
        
        # create linked list + hash of offsets, pointing at the first segment with this offset
        nextseg1, firstseg1 = Alignment.createOff2Seg(segments1)
        nextseg2, firstseg2 = Alignment.createOff2Seg(segments2)
        
        newSegments = []
        
        # for each offset from the two lists of segments
        for off in Alignment.genSumKeys(firstseg1, firstseg2):
            
            # construct generators of segments of offset off
            gen1 = Alignment.genSegments(segments1, off, nextseg1, firstseg1)
            gen2 = Alignment.genSegments(segments2, off, nextseg2, firstseg2)
            
            seg1 = gen1.next()[:]
            seg2 = gen2.next()[:]
            
            # for all segments with this offset
            while True:
                
                # calculate the segment before the intersection
                before = Alignment.segBefore(seg1, seg2 , SEG_START1, SEG_END1)
                Alignment.segAppend(newSegments, before)
                
                # calculate intersection of the two segments
                common = Alignment.segCommon(seg1, seg2)
                common[SEG_SCORE] = seg1[SEG_SCORE]+seg2[SEG_SCORE]
                common[SEG_IDENT] = seg1[SEG_IDENT]+seg2[SEG_IDENT]
                Alignment.segAppend(newSegments, common)
                
                # calculate the area after the common seg
                # operating on the copies, so no harm to the original list of segments
                seg1[SEG_START1] = max(seg1[SEG_START1], common[SEG_END1]+1)
                seg1[SEG_START2] = max(seg1[SEG_START2], common[SEG_END2]+1)
                seg2[SEG_START1] = max(seg2[SEG_START1], common[SEG_END1]+1)
                seg2[SEG_START2] = max(seg2[SEG_START2], common[SEG_END2]+1)
                
                
                if seg1[SEG_END1] < seg2[SEG_END1]:
                    if Debug.debug: assert Alignment.segEmpty(seg1)
                    seg1 = gen1.next()[:]
                else:
                    if Debug.debug: assert Alignment.segEmpty(seg2)
                    if seg2 == SEG_TERMINAL:
                        break
                    seg2 = gen2.next()[:]
                    
        return newSegments
       
    merge_segments = staticmethod(merge_segments)
                    
    def merge(self, align):
        
        # debug
        if Debug.debug:
            _debug_score_coverage = [ self.scoreCoverageCountSeq1(), self.scoreCoverageCountSeq2() ]
            _align_score_coverage = [ align.scoreCoverageCountSeq1(), align.scoreCoverageCountSeq2() ]
            for i in xrange(0, 2):
                assert len(_debug_score_coverage[i]) == len(_align_score_coverage[i])
                for j in xrange(0, len(_debug_score_coverage[i])):
                    _debug_score_coverage[i][j] +=  _align_score_coverage[i][j]
            _align_score_coverage = None
            
        # sanity check
        if self.seq1 != align.seq1 or self.seq2 != align.seq2:
            raise InvalidParameter, "Trying to merge different alignments (names don't match)"
            
        self.segments = Alignment.merge_segments(self.segments, align.segments)
        
        # debug
        if Debug.debug:
            _debug_score_coverage_after = [ self.scoreCoverageCountSeq1(), self.scoreCoverageCountSeq2() ]
            Alignment.assertCoverages(_debug_score_coverage, _debug_score_coverage_after)
                        
#
#    def _merge(self, align):
#        '''This private method merges the alignments, adding scores where the matches overlap.
#        Input: in align, no residue is covered by two segments (assures ordering)
#        After calling merge method this constraint is not met for self!'''
#        
#        print >>sys.stderr, "Ugly hack!"
#        for seg in self.segments:
#            seg[SEG_SCORE] = 1
#        
#        for seg in align.segments:
#            seg[SEG_SCORE] = 1
#            
#        Alignment.print_a_seg(self.segments, 196347, SEG_START1, SEG_END1)
#        Alignment.print_a_seg(align.segments, 196347, SEG_START1, SEG_END1)
#        Alignment.print_a_seg(self.segments, 196348, SEG_START1, SEG_END1)
#        Alignment.print_a_seg(align.segments, 196348, SEG_START1, SEG_END1)
#        print "self"; Alignment.print_a_seg(self.segments, 196366, SEG_START1, SEG_END1)
#        print "align"; Alignment.print_a_seg(align.segments, 196366, SEG_START1, SEG_END1)
#        
#        # debug
#        if (Debug.debug):
#            _debug_score_coverage = [ self.scoreCoverageCountSeq1(), self.scoreCoverageCountSeq2() ]
#            #_debug_score_coverage[0][0] += 1
#            _align_score_coverage = [ align.scoreCoverageCountSeq1(), align.scoreCoverageCountSeq2() ]
#            for i in xrange(0, 2):
#                assert len(_debug_score_coverage[i]) == len(_align_score_coverage[i])
#                for j in xrange(0, len(_debug_score_coverage[i])):
#                    _debug_score_coverage[i][j] +=  _align_score_coverage[i][j]
#            _align_score_coverage = None
#            
#        # sanity check
#        if self.seq1 != align.seq1 or self.seq2 != align.seq2:
#            raise InvalidParameter, "Trying to merge different alignments (names don't match)"
#            
#        # make sure that for align no matches start at the same position
#        # this is not true for self.assertSingleCoverage()
#        if Debug.debug:
#            align.assertSingleCoverage()
#            
#        # sorting the data
#        self.sort(SEG_START1)
#        align.sort(SEG_START1) # it's sorted according to end1 as well, because of single covarage
#        
#        # !!!ideally, validate method should be used, it's doing the same job!
#        # but it would have to be improved for the case when self is multiple covered (still "shadows" are single covered)
#        
#        i = 0
#        startj = 0
#        # segments which are result of fragmenting
#        brokenSegments = [] 
#        mergedSegments = [ False ] * len(align.segments)
#        while i < len(self.segments):
#            
#            # advance startj to the first segment overlapping with the new i
#            while startj < len(align.segments):
#                if align.segments[startj][SEG_END1] < self.segments[i][SEG_START1]:
#                    startj += 1
#                else:
#                    break
#            
#            # init j
#            j = startj
#            
#            while j < len(align.segments):
#                
#                # check whether there's an alignment to merge
#                if Alignment.segIntersect(self.segments[i], align.segments[j]):
#                    
#                    # mark the segment as merged
#                    mergedSegments[j] = True
#                    
#                    # calculate the new alignemnt
#                    segIntersect = Alignment.segCommon(self.segments[i], align.segments[j])
#                    assert segIntersect[SEG_START1] <= segIntersect[SEG_END1]
#                    
#                    # update the score values
#                    segIntersect[SEG_SCORE] = self.segments[i][SEG_SCORE] + align.segments[j][SEG_SCORE]
#                    
#                    # split the old alignment into three pieces: 
#                    # before merged, merged==segIntersect and after merged
#                    
#                    # after merged
#                    if self.segments[i][SEG_END1] > align.segments[j][SEG_END1]:
#                        score = self.segments[i][SEG_SCORE]
#                    else:
#                        score = align.segments[j][SEG_SCORE]
#                    segAfter = \
#                        [segIntersect[SEG_END1]+1, \
#                        segIntersect[SEG_END2]+1, \
#                        max(self.segments[i][SEG_END1], align.segments[j][SEG_END1]), \
#                        max(self.segments[i][SEG_END2], align.segments[j][SEG_END2]), \
#                        -1, \
#                        score]
#                    print "segAfter"; print segAfter
#                        
#                    print "self.segments[i]"; print self.segments[i]
#                    print "align.segments[j]"; print align.segments[j]
#                    print "segIntersect"; print segIntersect
#                    # before merged
#                    if self.segments[i][SEG_START1] < align.segments[j][SEG_START1]:
#                        score = self.segments[i][SEG_SCORE]
#                    else:
#                        score = align.segments[j][SEG_SCORE]
#                    segBefore = \
#                        [min(self.segments[i][SEG_START1], align.segments[j][SEG_START1]), \
#                        min(self.segments[i][SEG_START2], align.segments[j][SEG_START2]), \
#                        segIntersect[SEG_START1]-1, \
#                        segIntersect[SEG_START2]-1, \
#                        -1, \
#                        score]
#                    print "segBefore"; print segBefore
#                        
#                    # append merged segIntersect
#                    # actually, store it for later because 
#                    # it could be put somewhere further than i in the matrix, and
#                    # later inspected again, but we don't want align.segments[j]
#                    # intersect with it again
#                    assert Alignment.segNonEmpty(segIntersect)
#                    print "appending to broken segs"; print segIntersect
#                    brokenSegments.append(segIntersect)
#                    #segInsert(self.segments, segIntersect, SEG_START1)
#                    
#                    # store it for later
#                    # (could be added already, but this could make more work because indexes shift)
#                    if Alignment.segNonEmpty(segBefore):
#                        print "appending to broken segs 2"; print segBefore
#                        brokenSegments.append(segBefore)
#                        #segInsert(self.segments, segBefore, SEG_START1)
#                        
#                    # insert the new segment into appropriate place
#                    if Alignment.segNonEmpty(segAfter):
#                        # override the old segment, breaks the order!
#                        self.segments[i] = segAfter 
#                        # despite breaking the order we don't have to change i,
#                        # because only SEG_END1 of i is tested, which didn't changed
#                        #segInsert(self.segments, segAfter, SEG_START1)
#                    else:
#                        # well, time consuming...
#                        del self.segments[i]
#                        i -= 1 # restart the i loop
#                        break
#
#                if align.segments[j][SEG_START1] <= self.segments[i][SEG_END1]:
#                    j += 1
#                else:
#                    break
#                    
#            # advance i
#            i += 1
#            
#        # put broken segments into the segments
#        
#        # retrieve the order, broken by merged segments
#        self.sort(SEG_START1)
#        Alignment.sort_segments(brokenSegments, SEG_START1)
#        
#        # put the segments which were not merged by now
#        notMerged = []
#        for j in xrange(0, len(mergedSegments)):
#            if not mergedSegments[j]:
#                assert len(align.segments[j]) == SEG_SIZE
#                notMerged.append(align.segments[j])
#        Alignment.sort_segments(notMerged, SEG_START1)
#
#        print "self.segments"
#        Alignment.print_a_seg(self.segments, 196348, SEG_START1, SEG_END1)
#        print "brokenSegments"
#        Alignment.print_a_seg(brokenSegments, 196348, SEG_START1, SEG_END1)
#        print "notMerged"
#        Alignment.print_a_seg(notMerged, 196348, SEG_START1, SEG_END1)
#        
#        # put the postponed segments in the main list
#        self.segments = Alignment.merge_lists([self.segments, brokenSegments, notMerged], \
#            lambda seg1, seg2: seg1[SEG_START1] < seg2[SEG_START1])
#            
#        print >>sys.stderr, "Merged stuff!"
#        Alignment.print_a_seg(self.segments, 196347, SEG_START1, SEG_END1)
#        Alignment.print_a_seg(self.segments, 196348, SEG_START1, SEG_END1)
#        
#        # debug
#        if Debug.debug:
#            _debug_score_coverage_after = [ self.scoreCoverageCountSeq1(), self.scoreCoverageCountSeq2() ]
#            for i in xrange(0, 2):
#                assert len(_debug_score_coverage[i]) == len(_debug_score_coverage_after[i])
#                for j in xrange(0, len(_debug_score_coverage[i])):
#                    if not _debug_score_coverage[i][j] ==  _debug_score_coverage_after[i][j]:
#                        assert False, "pos "+str(i)+","+str(j)+" vb:"+str(_debug_score_coverage[i][j])+" va:"+str(_debug_score_coverage_after[i][j])
#
#    def merge(self, align):
#        return self._merge(align)

    def merge_lists(lists, isLess):
        '''Merges given (sorted) lists according to criteria. 
        Criteria gets 2 list elements and returns True if 1st one is smaller.
        Modifies "lists" parameter.'''
        
        # sanity check for non-empty lists
        for list_i in xrange(len(lists)-1, -1, -1):
            if len(lists[list_i]) == 0:
                del lists[list_i]

        # set the indices
        index = [ 0 ] * len(lists)
        merged = []
        while len(lists) > 0:
            # assume the smallest is the first one
            minElem_i = 0
            # maybe there's even smaller element
            for list_i in xrange(1, len(lists)):
                if isLess(lists[list_i][index[list_i]], lists[minElem_i][index[minElem_i]]):
                    minElem_i = list_i
            # append minimale element to the list
            merged.append(lists[minElem_i][index[minElem_i]])
            # push the iterator
            index[minElem_i] += 1
            if index[minElem_i] == len(lists[minElem_i]):
                del lists[minElem_i]
                del index[minElem_i]
            else:
                # sanity check
                if Debug.debug:
                    assert not isLess(lists[minElem_i][index[minElem_i]], lists[minElem_i][index[minElem_i]-1])
                
        return merged
        
    merge_lists = staticmethod(merge_lists)

    def sort(self, seg_coord):
        Alignment.sort_segments(self.segments, seg_coord)

#    def _sort_segments(segments, alist):
#        decorated = zip(alist, segments)
#        decorated.sort()
#        return [ seg for anitem, seg in decorated ]
#
#    _sort_segments = staticmethod(_sort_segments)

    def swap_coord(list, c1, c2):
        # swap coordinates
        if c1 != c2:
            for seg in list:
                tmp = seg[c1]
                seg[c1] = seg[c2]
                seg[c2] = tmp

    swap_coord = staticmethod(swap_coord)
    
    def sort_segments(list, coord):
        '''Sorts the list of lists in place, according to coord.'''
        assert Alignment.NO_SORT_REASON == 0
        Alignment.swap_coord(list, 0, coord)
        list.sort()
        Alignment.swap_coord(list, 0, coord)

    sort_segments = staticmethod(sort_segments)

    def sort_segments_order(list, coord):
        '''Sorts the list of lists in place, according to coord.'''
        order = [x for x in xrange(0, len(list))]
        Alignment.swap_coord(list, 0, coord)
        order.sort(lambda a, b: cmp(list[a], list[b]) )
        Alignment.swap_coord(list, 0, coord)
        return order

    sort_segments_order = staticmethod(sort_segments_order)

    def segCommon(seg1, seg2):
        '''Common part of the two alignments. No phase taken into account.'''
        return [max(seg1[SEG_START1], seg2[SEG_START1]), \
            max(seg1[SEG_START2], seg2[SEG_START2]), \
            min(seg1[SEG_END1], seg2[SEG_END1]), \
            min(seg1[SEG_END2], seg2[SEG_END2]), \
            -1, \
            -1]
            
    segCommon = staticmethod(segCommon)
        
    def segIntersect(seg1, seg2):
        '''True for intersecting segments (phase taken into account).'''
        if not Alignment.segInPhase(seg1, seg2):
            return False
        seg = Alignment.segCommon(seg1, seg2)
        return seg[SEG_START1]<= seg[SEG_END1] and seg[SEG_START2] <= seg[SEG_END2]
            
    segIntersect = staticmethod(segIntersect)
        
    def segInsert(segments, seg, coord):
        '''Inserts seg into segments preserving order according to coord. 
        Returns position where the element was inserted'''
        i = 0
        while i < len(segments):
            if segments[i][coord] > seg[coord]:
                break
            i += 1
        segments.insert(i, seg)
        # sanity checks
        assert i == 0 or segments[i-1][coord] <= segments[i][coord]
        assert i == len(segments)-1 or segments[i][coord] <= segments[i+1][coord]
        
        # return the position of the new element
        return i
        
    segInsert = staticmethod(segInsert)
        
    def segAppend(segments, seg):
        assert len(seg) == SEG_SIZE
        if seg[SEG_START1] <= seg[SEG_END1]:
            assert seg[SEG_START1]-seg[SEG_END1] == seg[SEG_START2]-seg[SEG_END2]
            segments.append(seg)

    segAppend = staticmethod(segAppend)
    
    def segNonEmpty(seg):
        assert len(seg) == SEG_SIZE
        assert seg[SEG_START1]-seg[SEG_END1] == seg[SEG_START2]-seg[SEG_END2]
        return seg[SEG_START1] <= seg[SEG_END1]

    segNonEmpty = staticmethod(segNonEmpty)
    
    def segEmpty(seg):
        assert len(seg) == SEG_SIZE
        return seg[SEG_START1] > seg[SEG_END1]

    segEmpty = staticmethod(segEmpty)
    
    def segInPhase(seg1, seg2):
        '''Calculates whether segments are in phase, i.e. prerequisite to overlapping
        (but doesn't check the overlap).'''
        return seg1[SEG_START1]-seg1[SEG_START2] == seg2[SEG_START1]-seg2[SEG_START2]

    segInPhase = staticmethod(segInPhase)
    
    def intersect(seg, i, align):
        '''Calculate the intersection between segment and alignment given.
        Index can be moved!'''
        
        new_segs = []
        
        # find the segment intersecting with my_seg, or the one right to my_seg
        while i < len(align):
                    
            # are segments "in phase"?
            if Alignment.segInPhase(seg, align[i]):
            #Alignment.start1(seg)-Alignment.start2(seg) == Alignment.start1(align[i])-Alignment.start2(align[i]):
            
                # calculate the intersection of seg and align[i]
                inter_start1, inter_start2, inter_end1, inter_end2, _ident, _score = Alignment.segCommon(seg, align[i])

                # is the intersection alignment non-empty?
                if inter_start1 <= inter_end1:
                    assert inter_start1 - inter_end1 == inter_start2 - inter_end2
                    
                    # add the intersecting segment to the list
                    new_segs.append([inter_start1, inter_start2, inter_end1, inter_end2, -1, -1])
 
            # advance to the next segment in the alignment
            if align[i][SEG_END1] <= seg[SEG_END1] or align[i][SEG_END2] <= seg[SEG_END2]:
                i += 1
            else:
                break
            
        return i, new_segs
        
    intersect = staticmethod(intersect)
    
    def validate_with_shadow(segments, shadow_segments):
        '''Increases identity wherever alignments intersect. Results
        in splitting alignments.'''
        new_segments = []
        
        # TODO 
        # 1) base intersection method based on this shadow
        # 2) the method is not working properly when the subsequent segments "cross"
        # !!!Sort the data before calculating the alignment!, probably according to start1, and end1
        
        i = 0
        j = 0
        while i < len(segments):
            
            # assure a total ordering on the alignment
            if not (i == len(segments)-1 \
                or (segments[i][SEG_START1] < segments[i+1][SEG_START1] and segments[i][SEG_START2] < segments[i+1][SEG_START2])):
                    print >> sys.stderr, "Segment cross",segments[i],segments[i+1]
            assert i == len(segments)-1 \
                or (segments[i][SEG_START1] < segments[i+1][SEG_START1])# TODO(2) and segments[i][SEG_START2] < segments[i+1][SEG_START2])
                
            seg = copy(segments[i])
            
            # iterate over intersected segments
            while j < len(shadow_segments):
                
                if not (j == len(shadow_segments)-1 \
                    or (shadow_segments[j][SEG_START1] < shadow_segments[j+1][SEG_START1] and shadow_segments[j][SEG_START2] < shadow_segments[j+1][SEG_START2])):
                        print >> sys.stderr, "Segment shadow cross",shadow_segments[j],shadow_segments[j+1]
                
                # assure a total ordering on the alignment
                assert j == len(shadow_segments)-1 \
                    or (shadow_segments[j][SEG_START1] < shadow_segments[j+1][SEG_START1])# TODO(2) and shadow_segments[j][SEG_START2] < shadow_segments[j+1][SEG_START2])

                shadow_seg = shadow_segments[j]
                
                # are alignemnts in phase?
                if seg[SEG_START1]-seg[SEG_START2] == shadow_seg[SEG_START1]-shadow_seg[SEG_START2]:
                    
                    # calculate the intersection of segments[i] and intersect[j]
                    inter_seg = Alignment.segCommon(seg, shadow_seg)
                    
                    # is the intersection alignment non-empty?
                    if inter_seg[SEG_START1] <= inter_seg[SEG_END1]:
                        # append what's before the intersection
                        Alignment.segAppend(new_segments, [seg[SEG_START1], seg[SEG_START2], inter_seg[SEG_START1]-1, inter_seg[SEG_START2]-1, seg[SEG_IDENT], seg[SEG_SCORE]])
                        # append the intersection
                        inter_seg[SEG_IDENT] = seg[SEG_IDENT]+1
                        inter_seg[SEG_SCORE] = seg[SEG_SCORE]
                        Alignment.segAppend(new_segments, inter_seg)
                        # mark the end of what was appended
                        seg[SEG_START1] = inter_seg[SEG_END1]+1
                        seg[SEG_START2] = inter_seg[SEG_END2]+1
                    
                if shadow_seg[SEG_END1] <= seg[SEG_END1] or shadow_seg[SEG_END2] <= seg[SEG_END2]:
                    j += 1
                else:
                    break
                    
            # append what's behind the intersection
            Alignment.segAppend(new_segments, seg)
            i += 1
                
        return new_segments
        
    validate_with_shadow = staticmethod(validate_with_shadow)
    
    def validate(self, aligns):
        '''Validates itself using other alignments. The number of alignments which 
        validated this alignment is stored in identity field. May result in splitting segments, but 
        leaves the alignment intact.
        '''
        self.assertSingleCoverage()
        
        # zero the identitiy field for the alignments
        for seg in self.segments:
            seg[SEG_IDENT] = 0

        for shadow in aligns:
            self.segments = Alignment.validate_with_shadow(self.segments, shadow.segments)

        
    def rescaleident(self, vmin, vmax):
        '''Changes the identity field proportionally to be between vmin and vmax
        '''
        # zero the identitiy field for the alignments
        maxident = 1
        for seg in self.segments:
            maxident = max(seg[SEG_IDENT], maxident)

        for seg in self.segments:
            seg[SEG_IDENT] = int(vmin + (vmax-vmin)*(1.0*seg[SEG_IDENT]/maxident))
            
    def _transform_segments(segments, add, mul):
        '''Linear transformation of the identity field
        '''
        # zero the identitiy field for the alignments
        for seg in segments:
            seg[SEG_IDENT] = int(round(mul*seg[SEG_IDENT] + add))

    _transform_segments = staticmethod(_transform_segments)
            
    def segTransitive(seg1, seg2):
        '''Calcs transitive segment.'''
        istart = max(seg1[SEG_START2], seg2[SEG_START1])
        iend = min(seg1[SEG_END2], seg2[SEG_END1])
        
        return [ seg1[SEG_START1] + (istart - seg1[SEG_START2]), \
            seg2[SEG_START2] + (istart - seg2[SEG_START1]), \
            seg1[SEG_END1] - (seg1[SEG_END2] - iend), \
            seg2[SEG_END2] - (seg2[SEG_END1] - iend), \
            -1, \
            min(seg1[SEG_SCORE], seg2[SEG_SCORE]) ]

    segTransitive = staticmethod(segTransitive)

    def segBreak(seg, next_seg, segments):
        Alignment.segAppend(segments, Alignment.segBefore(seg, next_seg, SEG_START1, SEG_END1))
        iseg = Alignment.segCommon(seg, next_seg)
        iseg[SEG_SCORE] = seg[SEG_SCORE] + next_seg[SEG_SCORE]
        Alignment.segAppend(segments, iseg)
        Alignment.segAppend(segments, Alignment.segAfterCommon(seg, next_seg, SEG_START1, SEG_END1))

    segBreak = staticmethod(segBreak)

    def removeOverlapSamePhaseOnePhase(gen):
        '''Remove overlapping segments of the same phase. Returns true if a segment was broken'''
        
        new_segments = []
        broke_segment = False
        
        # for all segments with this offset
        seg = gen.next()
        while True:
            if seg == SEG_TERMINAL:
                break

            next_seg = gen.next()
            
            assert seg[SEG_START1] <= next_seg[SEG_START1]
            assert seg[SEG_START1] != next_seg[SEG_START1] or seg[SEG_END1] <= next_seg[SEG_END1]
            assert Alignment.segInPhase(seg, next_seg) or next_seg == SEG_TERMINAL
                
            # is there an unhealthy overlap?
            if seg[SEG_END1] >= next_seg[SEG_START1]:
#                print "Breaking", seg, next_seg
                # break it down
                Alignment.segBreak(seg, next_seg, new_segments)
                # remember this historic event
                broke_segment = True
                # both segments has been consumed
                next_seg = gen.next()
            else:
                # no overlap - do it normally
                new_segments.append(seg)
                
            seg = next_seg
            
        return new_segments, broke_segment
                    
    removeOverlapSamePhaseOnePhase = staticmethod(removeOverlapSamePhaseOnePhase)

    def removeOverlapSamePhase(segments):
        '''Remove overlapping segments of the same phase'''
        
        new_segments = []
        segments.append(SEG_TERMINAL)
        Alignment.sort_segments(segments, SEG_START1)
        
        # without overlap checking!
        nextseg, firstseg = Alignment.createOff2Seg(segments, False)
        
        for off in firstseg:
            
            # construct generators of segments of offset off
            gen = Alignment.genSegments(segments, off, nextseg, firstseg)
            phase_segments, broke_segment = Alignment.removeOverlapSamePhaseOnePhase(gen)

            while broke_segment:
                Alignment.sort_segments(phase_segments, SEG_START1)
#                print "Rerun", phase_segments
                phase_segments, broke_segment = Alignment.removeOverlapSamePhaseOnePhase(Alignment.genAllSegments(phase_segments))
            
            new_segments.extend(phase_segments)
            
        return new_segments
        
    removeOverlapSamePhase = staticmethod(removeOverlapSamePhase)

    def _transitive(segments1, segments2):
        '''Calculates the transitive alignments between segments1 and segments2'''
        
        Alignment.sort_segments(segments1, SEG_START2)
        Alignment.sort_segments(segments2, SEG_START1)
        
        new_segments = []
        pending = []
        i2 = 0
        
        # for each of segments1
        for i1 in xrange(0, len(segments1)):
            
            seg1 = segments1[i1]
            
            # store in the queue all the segments seg2 which start before seg1 ends
            while i2 < len(segments2) and segments2[i2][SEG_START1] <= seg1[SEG_END2]:
                pending.append(segments2[i2])
                i2 += 1
        
            # inspect the queue
            for p in xrange(len(pending)-1, -1, -1):
                
                seg2 = pending[p]
                
                # is there a chance for seg2 to intersect with segments1[i1...]?
                if seg2[SEG_END1] < seg1[SEG_START2]:
                    # no - delete it
                    del pending[p]
                else:
                    # calculate the intersection
                    segTrans = Alignment.segTransitive(seg1, seg2)
                    Alignment.segAppend(new_segments, segTrans)
        
        # make sure that in the same phase there's no overlap
        new_segments = Alignment.removeOverlapSamePhase(new_segments)
        
        return new_segments
                    
    _transitive = staticmethod(_transitive)

    def transitive(a1, a2, overlap):
        
        # sanity check
        assert a1.seq2 == a2.seq1

        new_segments = Alignment._transitive(a1.segments, a2.segments)
        
        return Alignment(a1.seq1, a1.file1, a1.seq1_start, a1.seq1_end, \
            a2.seq2, a2.file2, a2.seq2_start, a2.seq2_end, \
            new_segments, overlap)
            
    transitive = staticmethod(transitive)

#    def transitive(a1, a2, overlap):
#        A  = Alignment
#        # sanity check
#        assert a1.seq2 == a2.seq1
#        
#        new_segments = []
#        
#        # sorting the data
#        a1.sort(SEG_START2)
#        a2.sort(SEG_START1)
#        
#        # iterate through the a1.segments
#        pos1 = 0
#        pos2 = 0
#        while pos1 < len(a1.segments) and pos2 < len(a2.segments):
#            
#            # create the intersecting alignment, possibly "negative" size
#            inter_start = max(A.start2(a1.segments[pos1]), A.start1(a2.segments[pos2]))
#            inter_end = min(A.end2(a1.segments[pos1]), A.end1(a2.segments[pos2]))
#            
#            # is it non-empty?
#            if inter_end - inter_start >= 0:
#                # create new transitive alignment
#                new_start1 = A.start1(a1.segments[pos1]) + (inter_start - A.start2(a1.segments[pos1]))
#                new_start2 = A.start2(a2.segments[pos2]) + (inter_start - A.start1(a2.segments[pos2]))
#                new_end1 = A.end1(a1.segments[pos1]) + (inter_end - A.end2(a1.segments[pos1]))
#                new_end2 = A.end2(a2.segments[pos2]) + (inter_end - A.end1(a2.segments[pos2]))                    
#                new_id = -1
#                new_score = min(A.score(a1.segments[pos1]), A.score(a2.segments[pos2]))
#                seg = [ new_start1, new_start2, new_end1, new_end2, new_id, new_score ]
#                new_segments.append(seg)
#            
#            # advance the pointer for which end lags behind
#            if A.end2(a1.segments[pos1]) < A.end1(a2.segments[pos2]):
#                pos1 += 1
#            else:
#                pos2 += 1
#            
#        return Alignment(a1.seq1, a1.file1, a1.seq1_start, a1.seq1_end, \
#            a2.seq2, a2.file2, a2.seq2_start, a2.seq2_end, \
#            new_segments, overlap)
#            
#    transitive = staticmethod(transitive)
#
#    def start1(seg):
#        return seg[0]
#    start1 = staticmethod(start1)
#    
#    def start2(seg):
#        return seg[1]
#    start2 = staticmethod(start2)
#       
#    def end1(seg):
#        return seg[2]
#    end1 = staticmethod(end1)
#        
#    def end2(seg):
#        return seg[3]
#    end2 = staticmethod(end2)
#       
#    def ident(seg):
#        return seg[4]
#    ident = staticmethod(ident)
#       
#    def score(seg):
#        return seg[5]
#    score = staticmethod(score)
#       
    def __str__(self):
        return 'Alignment of '+self.seq1+' against '+self.seq2+' with '+str(len(self.segments))+' segments'
        
    def _getVertices(self, seg_start, seg_end):
        '''Creates vertices, represented by index in the returned array, containing list of segments (for a given vertex).'''
        
        # results
        vertex = list()
        
        # the rightmost boundary of the of the segmnets
        right_boundary = -1
        
        # create a list
        for seg in self.segments:

            # does the new segment overlap with the boundary
            if seg[seg_start] > right_boundary:
                # no, doesn't overlap, gets a new id
                vertex.append(list())
                
            # append to the list, associated with a vertex
            vertex[-1].append(seg)
            
            # extend the right boundary
            right_boundary = max(right_boundary, seg[seg_end])
            
        return vertex
        
    def getVerticesSeq1(self):
        '''Creates vertices, represented by index in the returned array, containing list of segments (for a given vertex).'''
        
        Alignment.sort_segments(self.segments, SEG_START1)
        return self._getVertices(SEG_START1, SEG_END1)
        
    def getVerticesSeq2(self):
        '''Creates vertices, represented by index in the returned array, containing list of segments (for a given vertex).'''
        
        self.sort(SEG_START2)
        return self._getVertices(SEG_START2, SEG_END2)
        
    def segToString(seg):
        assert len(seg) == SEG_SIZE
        return str(seg[SEG_START1])+':'+str(seg[SEG_START2])+':'+str(seg[SEG_END1])+':'+str(seg[SEG_END2])
        
    segToString = staticmethod(segToString)
        
    def segmentsToVertices(vertices):
        '''Converts list of segments (result of setToString) into segments->vertices mapping'''
        map = dict()
        for i in xrange(0, len(vertices)):
            assert len(vertices) > 0
            for seg in vertices[i]:
                seg_str = Alignment.segToString(seg)
                assert not map.has_key(seg_str)
                map[seg_str] = i
        return map

    segmentsToVertices = staticmethod(segmentsToVertices)
    
    def _fragmentSegmentsDir(segments, seg_start, seg_end, assertCov = False):
        '''Divides overlapping segments the way no segment starts in the middle of other.
        Chops only the beginning of the segment.'''
        # append terminator
        segments.append(SEG_TERMINAL)

        # sort
        Alignment.sort_segments(segments, seg_start)
        if Debug.debug:
            assert segments[-1] == SEG_TERMINAL
            
        # list of segments which started, but were not finished
        pending = []
        new_segments = []
        for i in xrange(0, len(segments)):
            # for the new segment
            # investigate all pending ones
            seg = segments[i]
            if Alignment.segContainsPos(seg, 15218, SEG_START2, SEG_END2):
                print seg
            for j in xrange(len(pending)-1, -1, -1):
                # break the segment
                before = Alignment.segBefore(pending[j], seg, seg_start, seg_end)
                # append if it's not empty
                Alignment.segAppend(new_segments, before)
                # calc what is left
                after  = Alignment.segAfterBefore(pending[j], seg, seg_start, seg_end)
                # is there anyhing left?
                if Alignment.segEmpty(after):
                    del pending[j]
                else:
                    if assertCov: 
                        assert pending[j][seg_start] == seg[seg_start] and pending[j][seg_end] == seg[seg_end], str(pending[j]) +" "+ str(seg)
                    pending[j] = after
            # append the current segment to fragment it later
            pending.append(seg)
        
        assert segments[-1] == SEG_TERMINAL
        del segments[-1]
                    
        assert len(pending) == 1 and pending[0] == SEG_TERMINAL
        assert new_segments == [] or new_segments[-1] != SEG_TERMINAL
         
        return new_segments
        
    _fragmentSegmentsDir = staticmethod(_fragmentSegmentsDir)

    def segMirror(segments):
        '''In-place swapping of coordinates'''
        
        for seg in segments:
            x = seg[SEG_START1]
            seg[SEG_START1] = -seg[SEG_END1]
            seg[SEG_END1] = -x
            x = seg[SEG_START2]
            seg[SEG_START2] = -seg[SEG_END2]
            seg[SEG_END2] = -x
            
    segMirror = staticmethod(segMirror)


    def _fragmentSegmentsSeq(segments, seg_start, seg_end, assertCov = False):
        '''Divides overlapping segments the way no segment starts in the middle of other,
        for one od the sequences'''
        
        assert (len(segments) == 0 or segments[-1] != SEG_TERMINAL)

        new_segments = Alignment._fragmentSegmentsDir(segments, seg_start, seg_end, assertCov)
        
        # mirror segments
        Alignment.segMirror(new_segments)
        
        new_segments = Alignment._fragmentSegmentsDir(new_segments, seg_start, seg_end, assertCov)
        assert len(new_segments) == 0 or new_segments[-1] != SEG_TERMINAL
        
        # mirror segments back
        Alignment.segMirror(new_segments)
        
        return new_segments
        
    _fragmentSegmentsSeq = staticmethod(_fragmentSegmentsSeq)
    
    def _fragmentCheck(segments):
        pass
    _fragmentCheck = staticmethod(_fragmentCheck)
    
    def is_sorted_sublist(super, sub):
        '''Is sub a sublist of super? Both sorted.'''
        i = 0
        for v in sub:
            
            while i < len(super) and super[i] < v:
                i += 1
                
            if i == len(super) or super[i] != v:
                 return False
                 
        return True
            
    is_sorted_sublist = staticmethod(is_sorted_sublist)
    
    def sort_uniq(list):
        list.sort()
        for i in xrange(len(list)-1, 0, -1):
            if list[i] == list[i-1]:
                del list[i]
    sort_uniq = staticmethod(sort_uniq)
    
    
    def merge_with(list, to_merge):
        '''Modifies list, extending it with to_merge'''
        
        list.extend(to_merge)
        Alignment.sort_uniq(list)
        
    merge_with = staticmethod(merge_with)

    
    def translateBreakpoints(breakpoints, length, off, i):
        '''Translates breakpoints between segments'''
        
        # translation
        trans = [ x+off for x in breakpoints ]
        
        # filter those which are out of seg
        filt = [ x for x in trans if x > 0 and x < length ]
        
        return filt
        
    translateBreakpoints = staticmethod(translateBreakpoints)

    def segmentsPropagateBreaks(segments, i, breakpoints, seg_start, visited, neighbors, visitedOther, neighborsOther):
        '''Breaks the segment i.
        breakpoints[j] in {1..len(seg)-1}.
        Returns the list of new breakpoints introduced to a given segment.'''
        
        seg_end = seg_start+2
        seg = segments[i]
        segLen = (seg[seg_end] - seg[seg_start]) + 1
#        print "Entering depth", depth,  "segment", i, "seq", seg_start, "breakpoints", breakpoints
        
        # new breaking points found here
        new_breakpoints = breakpoints[:]
    
        # was segment visited?
        if visited[i] != None:
            
            # was the segment visited with these breakpoints?
            if Alignment.is_sorted_sublist(visited[i], new_breakpoints):
#                print "Leaving immediatelly"
                return []
    
        else:
            
            # never visited before
#            print "Never been here!"
            
            visited[i] = []
            
            # construct a list of breakpoints in this segment/sequence pair
            # based on neighbors
            for j in neighbors[i]:

                # breakpoint because of start of the segment
                bp = segments[j][seg_start] - seg[seg_start]
                
                if (bp > 0 and bp < segLen): 
                    new_breakpoints.append(bp)
                
                # breakpoint because of end of the segment
                bp = (segments[j][seg_end] - seg[seg_start]) + 1
                
                if (bp > 0 and bp < segLen): 
                    new_breakpoints.append(bp)
                
            # preserve the order
            Alignment.sort_uniq(new_breakpoints)
            
#            print "new_breakpoints",new_breakpoints
        
        # for each segment which intersects with this segment in this sequence
        
        # did we collect any new breakpoints?
        collected = True
        
        while collected and len(new_breakpoints) > 0:
            
            collected = False
        
            # merge the new breakpoints
            Alignment.merge_with(visited[i], new_breakpoints)
            
            for j in neighbors[i]:
                
                neighbor = segments[j]
                
                # difference between start of segments
                off = seg[seg_start] - neighbor[seg_start]
                
                # translate breakpoints from this segment to the other
                translatedBreakpoints = Alignment.translateBreakpoints(new_breakpoints, (neighbor[seg_end] - neighbor[seg_start]) + 1, off, j)
                
                # call recursively
                if len(translatedBreakpoints) > 0:
                    collectedBreakpoints = Alignment.segmentsPropagateBreaks(segments, j, translatedBreakpoints, (seg_start+1)%2, \
                                            visitedOther, neighborsOther, visited, neighbors)
            
                    if len(collectedBreakpoints) > 0:
                        collected = True
                        
                        # translate new breakpoints
                        collectedBreakpoints = Alignment.translateBreakpoints(collectedBreakpoints, segLen, -off, i)
                        
                        # update the information for the segment, because we can get back
                        # to node i through the recursion
                        Alignment.merge_with(visited[i], collectedBreakpoints)
                        
                        # merge with the new
                        Alignment.merge_with(new_breakpoints, collectedBreakpoints)
          
  
        if len(new_breakpoints) > 0: pass
        return new_breakpoints
            
    segmentsPropagateBreaks = staticmethod(segmentsPropagateBreaks)
    
                
                
#            oi = orderm1[i] + 1
#            while oi < len(order) and segments[order[oi]][seg_start] <= segments[i][seg_end]:
#                
#                # calculate the position of the breakpoint
#                bp = segments[order[oi]][seg_start] - segments[i][seg_end]
#                
#                # is bp non-trivial
#                if bp > 0:
#                    # is it not too far?
#                    assert bp <= segments[i][seg_end] - segments[i][seg_start]
#                    
#                    new_breakpoints.append(bp)
#
#                oi += 1
    
    def segmentNeighbors(segments, seg_start, seg_end, order):
        '''Define neighbors of each segment - those segments which overlap with it on the
        sequence determined by seg_start and seg_end. Segments is its own neighbor!'''
        
        neighbors = [ [i] for i in xrange(0, len(segments)) ] 
        pending = []
        
        # for each of segments1
        for i in order:
            
            seg = segments[i]
            
            # inspect the queue
            for p in xrange(len(pending)-1, -1, -1):
                
                seg2 = segments[pending[p]]
                
                # do they intersect?
                if seg2[seg_end] < seg[seg_start]:
                    # no - delete it
                    del pending[p]
                else:
                    # remember it as a neighbor
                    neighbors[pending[p]].append(i)
                    neighbors[i].append(pending[p])
                    
            pending.append(i)
            
        return neighbors

    segmentNeighbors = staticmethod(segmentNeighbors)
    
    def calculateOrderm1(order):
    	orderm1 = [ -1 ] * len(order)
        for i in xrange(0, len(order)):
            assert orderm1[order[i]] == -1
            orderm1[order[i]] = i
        return orderm1
        
    calculateOrderm1 = staticmethod(calculateOrderm1)

    def segBreakWith(seg, bps):
        '''Breaks a single segment with breakpoints, introducing a break -before- the positio specified in bp.'''
        new_segments = []
        last_bp = 0
    	for bp in bps:
            
            if Debug.debug:
                assert bp > 0
                assert bp < (seg[SEG_END1] - seg[SEG_START1]) + 1, str(bps)+" "+str((seg[SEG_END1] - seg[SEG_START1]) + 1)
            
            new_segments.append( \
                [ seg[SEG_START1] + last_bp, seg[SEG_START2] + last_bp, \
                    seg[SEG_START1] + bp - 1 , seg[SEG_START2] + bp - 1, \
                    seg[SEG_IDENT], seg[SEG_SCORE] ] \
                )
                
            last_bp = bp
            
        new_segments.append( \
            [ seg[SEG_START1] + last_bp, seg[SEG_START2] + last_bp, \
                seg[SEG_END1] , seg[SEG_END2], \
                seg[SEG_IDENT], seg[SEG_SCORE] ] \
            )
            
        return new_segments
            
            
    segBreakWith = staticmethod(segBreakWith)
    
        
    def segmentsBreakWith(segments, bpList):
        '''Breaks segments in the places specified by list'''
        new_segments = []
        
    	for i in xrange(0, len(segments)):
            new_segments.extend(Alignment.segBreakWith(segments[i], bpList[i]))
            
        return new_segments
        
    segmentsBreakWith = staticmethod(segmentsBreakWith)


    
    def _fragmentSegments(segments, assertCov = False):
        '''Divides overlapping segments the way no segment starts in the middle of other'''
#        print >> sys.stderr, "_fragmentSegments hack!"
#        new_segments = []
#        for seg in segments:
#            for i in xrange(seg[SEG_START1], seg[SEG_END1]+1):
#                new_segments.append([i, i+seg[SEG_START2]-seg[SEG_START1], i, i+seg[SEG_START2]-seg[SEG_START1], seg[SEG_IDENT], seg[SEG_SCORE] ] )
        
        Alignment.NO_SORT_REASON += 1
        
        order1 = Alignment.sort_segments_order(segments, SEG_START1)
        order2 = Alignment.sort_segments_order(segments, SEG_START2)
        #order1m1 = Alignment.calculateOrderm1(order1)
        #order2m1 = Alignment.calculateOrderm1(order2)

        neighbors1 = Alignment.segmentNeighbors(segments, SEG_START1, SEG_END1, order1)
        neighbors2 = Alignment.segmentNeighbors(segments, SEG_START2, SEG_END2, order2)
        
        visited1 = [ None for x in segments ]
        visited2 = [ None for x in segments ]

        # propagate breaks
        for i in xrange(0, len(segments)):
#            print "Seq 0"
            Alignment.segmentsPropagateBreaks(segments, i, [], SEG_START1, visited1, neighbors1, visited2, neighbors2)
#            print "Seq 1"
            Alignment.segmentsPropagateBreaks(segments, i, [], SEG_START2, visited2, neighbors2, visited1, neighbors1)
            
        # no need to propagate on the other sequence
        
        if Debug.debug:
            for i in xrange(0, len(visited1)):
                assert visited1[i] == visited2[i], str(visited1[i]) + " " + str(visited2[i])
                
        # break the segments according to the pattern suggested by visited1
        new_segments = Alignment.segmentsBreakWith(segments, visited1)

        Alignment.NO_SORT_REASON -= 1
        
#        new_segments = segments
#        new_segments = Alignment._fragmentSegmentsSeq(new_segments, SEG_START1, SEG_END1)
#        new_segments = Alignment._fragmentSegmentsSeq(new_segments, SEG_START2, SEG_END2)
#        
#        if assertCov: 
#            
#            Alignment._fragmentSegmentsSeq(new_segments, SEG_START1, SEG_END1, True)
#            Alignment._fragmentSegmentsSeq(new_segments, SEG_START2, SEG_END2, True)
#
        return new_segments
        
    _fragmentSegments = staticmethod(_fragmentSegments)
    
    def fragmentSegments(self, assertCov = False):
        '''Divides segments the way no segment starts in the middle of other'''
        self.segments = Alignment._fragmentSegments(self.segments, assertCov)
        
    def assertCoverages(cov1, cov2):
        for i in xrange(0, 2):
            assert len(cov1[i]) == len(cov2[i])
            for j in xrange(0, len(cov1[i])):
                if not cov1[i][j] ==  cov2[i][j]:
                    assert False, "pos "+str(i)+","+str(j)+" vb:"+str(cov1[i][j])+" va:"+str(cov2[i][j])
                    
    assertCoverages = staticmethod(assertCoverages)
            
    def assertCoveragesGE(cov1, cov2):
        '''cov1 is greater than cov2'''
        for i in xrange(0, 2):
            assert len(cov1[i]) == len(cov2[i])
            for j in xrange(0, len(cov1[i])):
                assert not (cov1[i][j] == 0 and cov2[i][j] > 0), "pos "+str(i)+","+str(j)+" vb:"+str(cov1[i][j])+" va:"+str(cov2[i][j])
                    
    assertCoveragesGE = staticmethod(assertCoveragesGE)
    
    def findMatch(segments, pos1):
            
        for i in xrange(0, len(segments)):
            seg = segments[i]
            if seg[SEG_START1] <= pos1 and seg[SEG_END1] >= pos1:
                print >> sys.stderr, "found! ",seg, seg[SEG_START1]-seg[SEG_START2]
                    
    findMatch = staticmethod(findMatch)
    
    def segmentsBreakpointsSeqDir(segments, order, seg_start, seg_end, breakpoints):
        '''Defines breaking points for the segment, given sequence (seg_start) and order.'''

        breakpoints = [] * len(order)
        
        # create a list
        for i in xrange(0, len(order)):
            
            # for all segments which start before the one ends
            j = i+1
            while j<len(order) and segments[order[j]][seg_start] <= segments[order[i]][seg_end]:
                
                # define a breakpoint in this segment
                bp = segments[order[i]][seg_start]-segments[order[j]][seg_start]
                
                # we are interested in >0 breakpoints
                if bp > 0:
                    breakpoints[order[i]].append(bp)
                    
                j += 1
                
        return breakpoints

    segmentsBreakpointsSeqDir = staticmethod(segmentsBreakpointsSeqDir)
        
    def segmentsBreakpointsSeq(segments, seg_start, seg_end):
        '''Defines breaking points for the segment, given sequence (seg_start) and order.'''
        
        breakpoints.appAlignment.segmentsBreakpointsSeqDir(segments, seg_start, seg_end)
        
        # mirror segments
        Alignment.segMirror(new_segments)
        
        # mirror segments
        Alignment.segMirror(new_segments)
        
        return breakpoints

    segmentsBreakpointsSeq = staticmethod(segmentsBreakpointsSeq)
    
    def realign(self):
        
        # TODO: get rid of "segToString" - use vertex numbers
        if Debug.debug:
            _debug_score_coverage = [ self.scoreCoverageCountSeq1(), self.scoreCoverageCountSeq2() ]
            
        # fragment segments
        self.fragmentSegments(Debug.debug)
        assert len(self.segments) == 0 or self.segments[-1] != SEG_TERMINAL

        if Debug.debug:
            _debug_score_coverage_after = [ self.scoreCoverageCountSeq1(), self.scoreCoverageCountSeq2() ]
            Alignment.assertCoverages(_debug_score_coverage, _debug_score_coverage_after)
            _debug_score_coverage = _debug_score_coverage_after = None

        # create a mapping segments->vertices
        vertices1 = self.getVerticesSeq1()
        vertices2 = self.getVerticesSeq2()
        s_to_v1 = Alignment.segmentsToVertices(vertices1)
        s_to_v2 = Alignment.segmentsToVertices(vertices2)
        
        # create the graph
        graph = {}
        for seg in self.segments:
            v1 = s_to_v1[Alignment.segToString(seg)]
            v2 = s_to_v2[Alignment.segToString(seg)]
            if not graph.has_key(v1):
                graph[v1] = []
            graph[v1].append(v2)
            
        # the algorithm
        partition, mis1, mis2 = bipartiteMatch(graph)
        
        # convert the graph (it's reversed!) back to alignment
        new_segments = []
        for v2, v1 in partition.iteritems():
            # intersect
            selected_seg = None
            for seg in vertices1[v1]:
                if vertices2[v2].count(seg) == 1:
                    # assert selected_seg == None, 'v1 '+str(v1)+' v2 '+str(v2)+' vertices1[v1] '+str(vertices1[v1])+' vertices2[v2] '+str(vertices2[v2])+' graph[v1] '+str(graph[v1])
                    if selected_seg == None or seg[SEG_SCORE] > selected_seg[SEG_SCORE]: 
                        selected_seg = seg
        
            assert selected_seg != None
            new_segments.append(selected_seg)
            
#        for i in xrange(0, len(vertices1)):
#            for seg in vertices1[i]:
#                if seg[SEG_START1] >= 229179 and seg[SEG_END1] <= 229179:
#                    print >> sys.stderr, "segment ",seg
        self.segments = new_segments
            
        # some sanity checks
        if Debug.debug:
            self.assertSingleCoverage()
            assert self.remove_overlap() == 0
            
#            print >> sys.stderr, "after"
#            Alignment.findMatch(self.segments, 1102932)

    def segmentsSetValue(segments, coord, val):
    	for seg in segments:
            seg[coord] = val
            
    segmentsSetValue = staticmethod(segmentsSetValue)
    
    def countResidues(self, minident):
        '''Counts number of residues with the minimum identity level specified'''
        return Alignment._countResidues(self.segments, minident)

    def _countResidues(segments, minident):
        '''Counts number of residues with the minimum identity level specified'''
        count = 0
    	for seg in segments:
            assert seg[SEG_IDENT] >= 0
            if seg[SEG_IDENT] >= minident:
                length = (seg[SEG_END1] - seg[SEG_START1]) + 1
                assert length > 0
                count += length
                
        return count
                
    _countResidues = staticmethod(_countResidues)

    def _resolveOverlap(segments, _start):
        '''Checks whether there are 2 different segments which (partially) overlap (including phase).'''
        # prepare coord
        start1 = _start
        start2 = (_start+1)%2
        end1 = start1+2
        end2 = start2+2
        # sort
        Alignment.sort_segments(segments, start1)
        # scan segments
        for i in xrange(0, len(segments)):
            phase = segments[i][start1] - segments[i][start2]
            # scan the other segments
            j = i+1
            while j < len(segments) and segments[j][start1] <= segments[i][end1]:
                assert segments[i][start1]-segments[i][start2] != segments[j][start1]-segments[j][start2]
                j += 1
                
    _resolveOverlap = staticmethod(_resolveOverlap)

    def print_a_seg(segments, pos, seg_start, seg_end):
        '''Prints a segment having a position pos'''
        for seg in segments:
            if pos >= seg[seg_start] and pos <= seg[seg_end]:
                print seg

    print_a_seg = staticmethod(print_a_seg)
                
    def segContainsPos(seg, pos, seg_start, seg_end):
        '''Prints a segment having a position pos'''
        return pos >= seg[seg_start] and pos <= seg[seg_end]
                
    segContainsPos = staticmethod(segContainsPos)

    def _regionsSane(regions):
        '''sanity check'''
        for i in xrange(0, len(regions)):
            assert len(regions[i]) == 2
            assert regions[i][0] <= regions[i][1]
        for i in xrange(0, len(regions)-1):
            assert regions[i][1] < regions[i+1][0], str(regions[i])+" "+str(regions[i+1])

    _regionsSane = staticmethod(_regionsSane)

    def _regionsCover(regions, segments, start, end):
        '''Returns the number of residues from regions (pairs of positions)
        in the given sequence of alignment. Alignment can be overlapping, regions can't.
        Residues are counted once, even though they might be covered by alignment
        more than once.'''
        # sort
        Alignment.sort_segments(segments, start)
        regions.sort()
        
        # sanity check
        Alignment._regionsSane(regions)

        # main loop
        rc = 0
        i = 0
    	for r in regions:
            # advance pointer
            while i < len(segments) and segments[i][end] < r[0]:
                i += 1
                
            starti = i

            # gather covered positions
            d = dict()
            while i < len(segments) and segments[i][start] <= r[1]:
                for p in xrange(max(r[0], segments[i][start]), min(r[1], segments[i][end])+1):
                    d[p] = None
                i += 1
            
            # sum up covered region
            for _g in d.iterkeys():
                rc += 1
                
            # go back with the pointer
            i = starti
            
        return rc
            
    _regionsCover = staticmethod(_regionsCover)
    
    def _regionsReportIdent(regions, segments, start, end):
        '''Reports maximal identity levels in the regions.'''
        # sort
        Alignment.sort_segments(segments, start)
        regions.sort()
        
        # sanity check
        Alignment._regionsSane(regions)

        # main loop
        rc = 0
        i = 0
        for r in regions:
            ident = dict()
            
            for p in xrange(r[0], r[1]+1):
                ident[p] = 0
                
            for seg in segments:
                for p in xrange(max(r[0], seg[start]), min(r[1], seg[end])+1):
                    if ident[p] < seg[SEG_IDENT]:
                        ident[p] = seg[SEG_IDENT]
                    
            for p in xrange(r[0], r[1]+1):
                print p, ident[p]
                
                
            
    _regionsReportIdent = staticmethod(_regionsReportIdent)
    
    def regionsLength(regions):
        
        regions.sort()
        
        # sanity check
        Alignment._regionsSane(regions)

        rlen = 0
        for i in xrange(0, len(regions)):
            rlen +=  (regions[i][1] - regions[i][0]) + 1
            
        return rlen
        
    regionsLength = staticmethod(regionsLength)
    
    def regAppend(regions, r):
        '''Appends non-empty region'''
    	if r[0] <= r[1]:
            regions.append(r)
            
    regAppend = staticmethod(regAppend)


    def _regionsStickyMerge(regions):
        '''Megers regions which overlap and coincide,
        taking into account regions annotation (coord 2 up)'''
        regions.sort()
        regions_new = []
        ext_reg = regions[0][:]
        for reg in regions[1:]:
            if ext_reg[1] < reg[0]-1 or ext_reg[2:] != reg[2:]:
                Alignment.regAppend(regions_new, ext_reg)
                ext_reg = reg[:]
            else:
                # extension
                ext_reg[1] = max(ext_reg[1], reg[1])

        Alignment.regAppend(regions_new, ext_reg)

        return regions_new

    _regionsStickyMerge = staticmethod(_regionsStickyMerge)
    
    def _regionsMerge(regions):
        '''Megers regions which overlap'''
        regions.sort()
        regions_new = []
        reg = [-1, -2]
        i = 0
        while i < len(regions):
            if reg[1] <  regions[i][0]:
                if reg[0] <= reg[1]:
                    regions_new.append(reg)
                reg = regions[i][:]
            else:
                reg[1] = max(reg[1], regions[i][1])
            i += 1

        regions_new.append(reg)

        return regions_new

    _regionsMerge = staticmethod(_regionsMerge)
    
    def regSegIntersect(reg, seg, start, end):
    	skipB = max(reg[0], seg[start]) - seg[start]
        skipE = seg[end] - min(reg[1], seg[end])
        
        return [ seg[0] + skipB, \
            seg[1] + skipB, \
            seg[2] - skipE, \
            seg[3] - skipE, \
            seg[4], \
            seg[5] ]
            
    regSegIntersect = staticmethod(regSegIntersect)

    def _regionsFilter(regions, segments, start, end):
        '''Filters the alignment to regions.'''
        
        # sort
    	regions.sort()
        Alignment.sort_segments(segments, start)
        
        # sanity check
        Alignment._regionsSane(regions)

        # init
        news = []
        pending = []
        i = 0
        
        for r in regions:
                        
            # store in the queue all the segments seg2 which start before seg1 ends
            while i < len(segments) and segments[i][start] <= r[1]:
                pending.append(segments[i])
                i += 1
        
            # inspect the queue
            for p in xrange(len(pending)-1, -1, -1):
                
                seg = pending[p]
                
                # is there a chance for seg2 to intersect with segments1[i1...]?
                if seg[SEG_END1] < r[0]:
                    # no - delete it
                    del pending[p]
                else:
                    # calculate the intersection
                    segTrans = Alignment.regSegIntersect(r, seg, start, end)
                    Alignment.segAppend(news, segTrans)
        
        return news
        
    _regionsFilter = staticmethod(_regionsFilter)
    
    def segRecalcAppend(segments, seg, superSeg, start, end):
        '''Append the segment after recalculating start2 and end2, with a few checks'''
        
        if seg[start] <= seg[end]:
            
            start2 = (start+1)%2
            end2 = start2+2
            
            seg[start2] = superSeg[start2] + (seg[start] - superSeg[start])
            seg[end2] = superSeg[end2] - (superSeg[end] - seg[end])
            
            assert superSeg[start] <= seg[start] and superSeg[end] >= seg[end]
            assert superSeg[start2] <= seg[start2] and superSeg[end2] >= seg[end2]
            
            Alignment.segAppend(segments, seg)
            
    segRecalcAppend = staticmethod(segRecalcAppend)

    def _segmentsDiff(segments1, segments2, start, end):
    	'''Residues of sequence1 aligned with ALIGN1 inconsistent with ALIGN2.
        Returns inconsistent segments. Both segments1 and segments2 can overlap.'''

        # inconsistent segments to store
        new_segments = []

        Alignment.sort_segments(segments1, start)
        Alignment.sort_segments(segments2, start)
        
        # some useful definitions
        start2 = (start+1) % 2
        end2 = start2+2
        
        # position in segments2
        i = 0 
        pending = []
        
        for seg1 in segments1:
            
            # prepare the inconsistency segment
            # there may be many *overlapping* inconsistent segments in segment2
            seg_inc = seg1[:]
            seg_inc[end] = seg_inc[start]-1 
            # we don't care about seg_inc[start2] and seg_inc[end2], this will be calculated later
            
            #store in the queue all the segments seg2 which start before seg1 ends
            while i < len(segments2) and segments2[i][start] <= seg1[end]:
                pending.append(segments2[i])
                i += 1
            
            # purge the queue
            for p in xrange(len(pending)-1, -1, -1):
                # is there a chance for seg2 to intersect with segments1[i1...]?
                if pending[p][end] < seg1[start]:
                    # no - delete it
                    del pending[p]
                    
            # for each overlapping segment
            for seg2 in pending:
                
                # if segments overlap and 
                # the offset the same?
                if  max(seg2[start], seg1[start]) <= min(seg2[end], seg1[end]) and \
                    seg1[start]-seg1[start2] != seg2[start]-seg2[start2]:
                        
                    # now - got the culprit - inconsistency
                    newStart = max(seg1[start], seg2[start])
                    newEnd = min(seg1[end], seg2[end])

                    # is the inconsistency a continuation of the old inconsistent segment?
                    # includes +1 if the new inconsistency starts JUST AFTER the old one
                    if seg_inc[end]+1 >= newStart:
                        # it can be only "later" segment
                        assert seg_inc[start] <= newStart
                        # join them
                        seg_inc[end] = max(seg_inc[end], newEnd)
                    else:
                        # no overlap between segments, append new seg
                        # recalculate start2, end2
                        # append
                        Alignment.segRecalcAppend(new_segments, seg_inc[:], seg1, start, end)
                        
                        # remember the new segment
                        seg_inc[start] = newStart
                        seg_inc[end] = newEnd
                        
                # end of the "for seg2" loop
                    
            # append the rest inconsistent segment
            Alignment.segRecalcAppend(new_segments, seg_inc[:], seg1, start, end)
            
            # end of the "for seg1" loop
            
        return new_segments
        
    _segmentsDiff = staticmethod(_segmentsDiff)
    
class InvalidParameter(Exception):
    "Wrong parameter"