#!/usr/bin/python

import unittest
import readblastz

from alignment import Alignment
from alignment import InvalidParameter
from alignment import SEG_START1
from alignment import SEG_END1
from alignment import SEG_START2
from alignment import SEG_END2

class ModTest(unittest.TestCase):
    
    def testtransitive(self):
        a1 = readblastz.readblastz('data/a_b.blastz', Alignment.DONT_ALLOW_OVERLAP)
        a2 = readblastz.readblastz('data/b_c.blastz', Alignment.DONT_ALLOW_OVERLAP)
        a3 =  Alignment.transitive(a1, a2, Alignment.DONT_ALLOW_OVERLAP)
        self.assertEqual(a3.segments, [[210, 10, 220, 20, -1, 1], [230, 30, 240, 40, -1, 1], [290, 90, 300, 100, -1, 1], [410, 410, 420, 420, -1, 2], [430, 430, 440, 440, -1, 3], [450, 450, 460, 460, -1, 4]])
        
    def testintersect1(self):
        ab = readblastz.readblastz('data/a_b.blastz', Alignment.DONT_ALLOW_OVERLAP)
        seg = [250, 50, 435, 235, -1, -1]
        i, ab_intersect =  Alignment.intersect(seg, 0, ab.segments)
        self.assertEqual(ab_intersect, [[250, 50, 300, 100, -1, -1], [400, 200, 420, 220, -1, -1], [430, 230, 435, 235, -1, -1]])
        assert i == 2
        
    def testintersect2(self):
        ab = readblastz.readblastz('data/a_b.blastz', Alignment.DONT_ALLOW_OVERLAP)
        seg = [250, 50, 260, 60, -1, -1]
        i, ab_intersect =  Alignment.intersect(seg, 0, ab.segments)
        self.assertEqual(ab_intersect, [[250, 50, 260, 60, -1, -1]])
        self.assertEqual(i, 0)
        
    def testvalidate_with_shadow1(self):
        ab = readblastz.readblastz('data/a_b.blastz', Alignment.DONT_ALLOW_OVERLAP)
        seg = [250, 50, 435, 235, 0, 0]
        ab_validated =  Alignment.validate_with_shadow(ab.segments, [seg])
        self.assertEqual(ab_validated[:5], [[210, 10, 249, 49, 0, 1], [250, 50, 300, 100, 1, 1], [400, 200, 420, 220, 1, 2], [430, 230, 435, 235, 1, 3], [436, 236, 440, 240, 0, 3]])
        self.assertEqual(ab_validated[5:], ab.segments[3:])
        
    def testvalidate_with_shadow2(self):
        ab = readblastz.readblastz('data/a_b.blastz', Alignment.DONT_ALLOW_OVERLAP)
        seg = [250, 50, 260, 60, -1, -1]
        ab_validated =  Alignment.validate_with_shadow(ab.segments, [seg])
        self.assertEqual(ab_validated[:3], [[210, 10, 249, 49, 0, 1], [250, 50, 260, 60, 1, 1], [261, 61, 300, 100, 0, 1]])
        self.assertEqual(ab_validated[3:], ab.segments[1:])
        
    def testvalidate_with_shadow3(self):
        ab = readblastz.readblastz('data/a_b.blastz', Alignment.DONT_ALLOW_OVERLAP)
        seg = [250, 50, 435, 235, 0, 0]
        ab_validated =  Alignment.validate_with_shadow([seg], ab.segments)
        self.assertEqual(ab_validated, [[250, 50, 300, 100, 1, 0], [301, 101, 399, 199, 0, 0], [400, 200, 420, 220, 1, 0], [421, 221, 429, 229, 0, 0], [430, 230, 435, 235, 1, 0]])
        
    def testvalidate_with_shadow4(self):
        ab = readblastz.readblastz('data/a_b.blastz', Alignment.DONT_ALLOW_OVERLAP)
        seg1 = [250, 50, 435, 235, 0, 0]
        seg2 = [251, 50, 436, 235, 0,0]
        ab_validated =  Alignment.validate_with_shadow([seg1], [seg2])
        self.assertEqual(ab_validated, [seg1])
        ab_validated =  Alignment.validate_with_shadow([seg2], [seg1])
        self.assertEqual(ab_validated, [seg2])
        
    def testvalidate_with_shadow5(self):
        pass # test invalidate because the creation of an alignment doesn't allow 
#        align1 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
#            [ [1, 101, 20, 120, -1, 1], [10, 110, 15, 115, -1, 2] ])
#        align2 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
#            [ [10, 110, 15, 115, -1, 2] ])
#        print align1.segments
#        print align2.segments
#        ab_validated =  Alignment.validate_with_shadow(align1.segments, align2.segments)
#        self.assertEquals(ab_validated, [ [1, 101, 20, 120, 0, 1], [10, 110, 15, 115, 1, 2] ])
        
    def testremove_overlap1(self):
        align = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [1, 11, 10, 20, 0, 0], [9, 20, 19, 30, 0, 1] ])
        align.assertSingleCoverage()
#        self.assertEquals(align.segments, [ [9, 20, 19, 30, 0, 1] ])
        
    def testremove_overlap2(self):
        align = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [1, 11, 10, 20, 0, 1], [9, 20, 19, 30, 0, 0] ])
        align.assertSingleCoverage()
#        self.assertEquals(align.segments, [ [1, 10, 10, 20, 0, 1] ])
        
    def testremove_overlap3(self):
        align = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [1, 11, 10, 20, 0, 1], [11, 20, 21, 30, 0, 0] ])
        align.assertSingleCoverage()
#        self.assertEquals(align.segments, [ [1, 10, 10, 20, 0, 1] ])
        
    def testremove_overlap4(self):
        align = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [1, 11, 10, 20, 0, 0], [11, 20, 21, 30, 0, 1] ])
        align.assertSingleCoverage()
#        self.assertEquals(align.segments, [ [11, 20, 21, 30, 0, 1] ])
        
    def testmerge1(self):
        align1 = Alignment("s8", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [1, 101, 10, 110, -1, 1] ])
        align2 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [1, 101, 10, 110, -1, 1] ])
        # exception expected, names don't fit
        try:
            align1.merge(align2)
        except InvalidParameter:
            pass
        else:
            self.fail()
        
    def testmerge2(self):
        align1 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [1, 101, 10, 110, 0, 1] ])
        align2 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [1, 101, 10, 110, 0, 2] ])
        align1.merge(align2)
        self.assertEquals(align1.segments, [ [1, 101, 10, 110, 0, 3] ])
        
    def testmerge3(self):
        align1 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [20, 120, 40, 140, 0, 2] ])
        align2 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [30, 130, 50, 150, 0, 1] ])
        align1.merge(align2)
        self.assertEquals(align1.segments, [ [20, 120, 29, 129, 0, 2], [30, 130, 40, 140, 0, 3], [41, 141, 50, 150, 0, 1] ])
        
    def testmerge4(self):
        align1 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [70, 170, 90, 190, 0, 2] ])
        align2 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [60, 160, 80, 180, 0, 1] ])
        align1.merge(align2)
        self.assertEquals(align1.segments, [ [60, 160, 69, 169, 0, 1], [70, 170, 80, 180, 0, 3], [81, 181, 90, 190, 0, 2] ])
        
    def testmerge5(self):
        align1 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [100, 200, 130, 230, 0, 2] ])
        align2 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [110, 210, 120, 220, 0, 1] ])
        align1.merge(align2)
        self.assertEquals(align1.segments, [ [100, 200, 109, 209, 0, 2], [110, 210, 120, 220, 0, 3], [121, 221, 130, 230, 0, 2] ])
        
    def testmerge6(self):
        align1 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [110, 210, 120, 220, 0, 1] ])
        align2 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [100, 200, 130, 230, 0, 2] ])
        align1.merge(align2)
        self.assertEquals(align1.segments, [ [100, 200, 109, 209, 0, 2], [110, 210, 120, 220, 0, 3], [121, 221, 130, 230, 0, 2] ])

    def testmerge7(self):
        align1 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [180, 280, 190, 290, 0, 1] ])
        align2 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [180, 281, 190, 291, 0, 2] ])
        align1.merge(align2)
        self.assertEquals(align1.segments, [ [180, 280, 190, 290, 0, 1], [180, 281, 190, 291, 0, 2] ])
        
    def testmerge8(self):
        # assures sorting of align2
        align1 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [100, 200, 130, 230, 0, 2] ])
        align2 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [ [510, 610, 520, 620, 0, 1], [110, 210, 120, 220, 0, 1] ])
        align1.merge(align2)
        self.assertEquals(align1.segments, [ [100, 200, 109, 209, 0, 2], [110, 210, 120, 220, 0, 3], [121, 221, 130, 230, 0, 2], [510, 610, 520, 620, 0, 1] ])
        
        
    def testmerge9(self):
        # and empty alignments...
        align1 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [  ])
        align2 = Alignment("s1", "None", 1, 1000, "s2", "None", 1, 1000, \
            [  ])
        align1.merge(align2)
        self.assertEquals(align1.segments, [  ])
        
    def testmerge10(self):
        align1 = Alignment("s1", "None", 1, 200000, "s2", "None", 1, 200000, \
            [ [196342, 132610, 196348, 132616, 0, 1], [196366, 132634, 196387, 132655, 0, 1] ])
        align2 = Alignment("s1", "None", 1, 200000, "s2", "None", 1, 200000, \
            [ [196339, 132607, 196347, 132615, 0, 1], [196348, 132616, 196387, 132655, 0, 1] ])
        align1.merge(align2)
        self.assertEquals(align1.segments, [ [196339, 132607, 196341, 132609, 0, 1], [196342, 132610, 196347, 132615, 0, 2], [196348, 132616, 196348, 132616, 0, 2], [196349, 132617, 196365, 132633, 0, 1], [196366, 132634, 196387, 132655, 0, 2] ])
        
    def testmerge11(self):
        align1 = Alignment("s1", "None", 1, 200000, "s2", "None", 1, 200000, \
            [ [196342, 132610, 196348, 132616, 0, 1], [196366, 132634, 196387, 132655, 0, 1] ])
        align2 = Alignment("s1", "None", 1, 200000, "s2", "None", 1, 200000, \
            [ [196348, 132616, 196387, 132655, 0, 1] ])
        align1.merge(align2)
        self.assertEquals(align1.segments, [ [196342, 132610, 196347, 132615, 0, 1], [196348, 132616, 196348, 132616, 0, 2], [196349, 132617, 196365, 132633, 0, 1], [196366, 132634, 196387, 132655, 0, 2] ])
        
    def testcoverageCount1(self):
        align1 = Alignment("s1", "None", 1, 5, "s2", "None", 1, 5, \
            [ [1, 2, 3, 4, -1, -1] ])
        cov = align1.coverageCountSeq1()
        self.assertEquals(cov, [ 0, 1, 1, 1, 0, 0 ] )
        cov = align1.coverageCountSeq2()
        self.assertEquals(cov, [ 0, 0, 1, 1, 1, 0 ] )
        
    def testsegInsert1(self):
        segments = []
        seg = [0, 1, 2, 3, 4, 5]
        Alignment.segInsert(segments, seg, 0)
        self.assertEquals(segments, [ seg ])
        
    def testsegInsert2(self):
        seg1 = [0, 1, 2, 3, 4, 5]
        segments = [ seg1 ]
        seg2 = [-1, 1, 2, 3, 4, 5]
        Alignment.segInsert(segments, seg2, 0)
        self.assertEquals(segments, [ seg2, seg1 ])
        
    def testsegInsert3(self):
        seg1 = [0, 1, 2, 3, 4, 5]
        segments = [ seg1 ]
        seg2 = [1, 1, 2, 3, 4, 5]
        Alignment.segInsert(segments, seg2, 0)
        self.assertEquals(segments, [ seg1, seg2 ])
        
    def testsegInsert4(self):
        seg1 = [0, 1, 2, 3, 4, 5]
        seg3 = [2, 1, 2, 3, 4, 5]
        segments = [ seg1, seg3 ]
        seg2 = [1, 1, 2, 3, 4, 5]
        Alignment.segInsert(segments, seg2, 0)
        self.assertEquals(segments, [ seg1, seg2, seg3 ])
        
    def testsegInsert5(self):
        seg1 = [0, 1, 2, 3, 4, 5]
        segments = [ seg1 ]
        seg2 = [1, 0, 2, 3, 4, 5]
        Alignment.segInsert(segments, seg2, 1)
        self.assertEquals(segments, [ seg2, seg1 ])
        
    def testcoverageCount2(self):
        segs = [ [1, 2, 3, 4, -1, -1], [ 1, 1, 1, 1, -1, -1] ]
        align1 = Alignment("s1", "None", 1, 5, "s2", "None", 1, 5, segs[:])
        
        # a hack, because segments with overlapping positions has been removed
        align1.segments = segs[:]
        
        cov = align1.coverageCountSeq1()
        self.assertEquals(cov, [ 0, 2, 1, 1, 0, 0 ] )
        cov = align1.coverageCountSeq2()
        self.assertEquals(cov, [ 0, 1, 1, 1, 1, 0 ] )
        
    def test_getVertices1(self):
        segs = [  ]
        align1 = Alignment("s1", "None", 1, 5, "s2", "None", 1, 5, segs[:])
        
        self.assertEquals([], align1._getVertices(SEG_START1, SEG_END1))
        
    def test_getVertices2(self):
        segs = [ [1, 1, 3, 3, -1, -1] ]
        align1 = Alignment("s1", "None", 1, 5, "s2", "None", 1, 5, segs[:])
        
        self.assertEquals([segs], align1._getVertices(SEG_START1, SEG_END1))
        
    def test_getVertices3(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [4, 1, 5, 3, -1, -1]
        segs = [ seg1[:], seg2[:] ]
        align1 = Alignment("s1", "None", 1, 100, "s2", "None", 1, 100, segs[:], True)
        
        self.assertEquals([[seg1], [seg2]], align1._getVertices(SEG_START1, SEG_END1))
        
    def test_getVertices4(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [3, 1, 5, 3, -1, -1]
        segs = [ seg1, seg2 ]
        align1 = Alignment("s1", "None", 1, 5, "s2", "None", 1, 5, segs[:], True)
        
        self.assertEquals([[seg1, seg2]], align1._getVertices(SEG_START1, SEG_END1))
        
    def test_getVertices5(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [3, 1, 5, 3, -1, -1]
        seg3 = [5, 1, 7, 3, -1, -1]
        segs = [ seg1, seg2, seg3 ]
        align1 = Alignment("s1", "None", 1, 50, "s2", "None", 1, 50, segs[:], True)
        
        self.assertEquals([[seg1, seg2, seg3]], align1._getVertices(SEG_START1, SEG_END1))
        
    def test_getVertices6(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [1, 3, 3, 5, -1, -1]
        seg3 = [1, 5, 3, 7, -1, -1]
        segs = [ seg1, seg2, seg3 ]
        align1 = Alignment("s1", "None", 1, 50, "s2", "None", 1, 50, segs[:], True)
        
        self.assertEquals([[seg1, seg2, seg3]], align1._getVertices(SEG_START2, SEG_END2))
        
    def test_getVertices7(self):
        seg1 = [1, 1, 7, 3, -1, -1]
        seg2 = [3, 1, 5, 3, -1, -1]
        seg3 = [6, 1, 8, 3, -1, -1]
        segs = [ seg1, seg2, seg3 ]
        align1 = Alignment("s1", "None", 1, 50, "s2", "None", 1, 50, segs[:], True)
        
        self.assertEquals([[seg1, seg2, seg3]], align1._getVertices(SEG_START1, SEG_END1))
        
    def testmerge_lists1(self):
        self.assertEquals(Alignment.merge_lists([ [], [0] ], lambda a, b: a<b), [0])
        
    def testmerge_lists2(self):
        self.assertEquals(Alignment.merge_lists([ [0], [0, 1, 2] ], lambda a, b: a<b), [0, 0, 1, 2])
        
    def testmerge_lists3(self):
        self.assertEquals(Alignment.merge_lists([ [0, 1, 2], [0] ], lambda a, b: a<b), [0, 0, 1, 2])
        
    def testmerge_lists4(self):
        self.assertEquals(Alignment.merge_lists([ [0, 1, 2], [4], [5] ], lambda a, b: a<b), [0, 1, 2, 4, 5])
        
    def testmerge_lists5(self):
        self.assertEquals(Alignment.merge_lists([ [0, 1, 2], [5], [4] ], lambda a, b: a<b), [0, 1, 2, 4, 5])
        
    def testmerge_lists6(self):
        try:
            self.assertEquals(Alignment.merge_lists([ [0], [3, 1, 2] ], lambda a, b: a<b), [0, 0, 1, 2])
        except AssertionError:
            pass
        else:
            self.fail("Didn't spot that list is not sorted.")
        
    def testmerge_lists7(self):
        self.assertEquals(Alignment.merge_lists([ [ [0,1] ], [ [1,0] ]  ], lambda a1,a2: a1[0] < a2[0]), [ [0,1], [1,0] ])
        
    def testsegBefore1(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [1, 1, 3, 3, -1, -1]
        seg = Alignment.segBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [1, 1, 0, 0, -1, -1])
        
    def testsegBefore2(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [3, 3, 3, 3, -1, -1]
        seg = Alignment.segBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [1, 1, 2, 2, -1, -1])
        
    def testsegBefore3(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [4, 4, 4, 4, -1, -1]
        seg = Alignment.segBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [1, 1, 3, 3, -1, -1])
        
    def testsegBefore4(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [4, 4, 4, 4, -1, -1]
        seg = Alignment.segBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [1, 1, 3, 3, -1, -1])
        
    def testsegBefore5(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [5, 5, 5, 5, -1, -1]
        seg = Alignment.segBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [1, 1, 3, 3, -1, -1])
        
    def testsegBefore6(self):
        seg2 = [1, 1, 3, 3, -1, -1]
        seg1 = [3, 3, 3, 3, -1, -1]
        seg = Alignment.segBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [1, 1, 2, 2, -1, -1])
        
    def testsegBefore7(self):
        seg1 = [10, 10, 20, 20, -1, -1]
        seg2 = [15, 30, 25, 40, -1, -1]
        seg = Alignment.segBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [10, 10, 14, 14, -1, -1] )
        
    def testsegAfterBefore1(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [1, 1, 3, 3, -1, -1]
        seg = Alignment.segAfterBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [1, 1, 3, 3, -1, -1])
        
    def testsegAfterBefore2(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [3, 3, 3, 3, -1, -1]
        seg = Alignment.segAfterBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [3, 3, 3, 3, -1, -1])
        
    def testsegAfterBefore3(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [4, 4, 4, 4, -1, -1]
        seg = Alignment.segAfterBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assert_(Alignment.segEmpty(seg))
        
    def testsegAfterBefore4(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [4, 4, 4, 4, -1, -1]
        seg = Alignment.segAfterBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assert_(Alignment.segEmpty(seg))
        
    def testsegAfterBefore5(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [5, 5, 5, 5, -1, -1]
        seg = Alignment.segAfterBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assert_(Alignment.segEmpty(seg), str(seg))
        
    def testsegAfterBefore6(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [1, 1, 2, 2, -1, -1]
        seg = Alignment.segAfterBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [1, 1, 3, 3, -1, -1])
        
    def testsegAfterBefore7(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [1, 1, 5, 5, -1, -1]
        seg = Alignment.segAfterBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [1, 1, 3, 3, -1, -1])
        
    def testsegAfterBefore8(self):
        seg1 = [10, 10, 20, 20, -1, -1]
        seg2 = [15, 30, 25, 40, -1, -1]
        seg = Alignment.segAfterBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [15, 15, 20, 20, -1, -1] )
        
    def testsegAfterBefore9(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [4, 4, 4, 4, -1, -1]
        seg = Alignment.segAfterBefore(seg1, seg2,  SEG_START1, SEG_END1)
        self.assert_(Alignment.segEmpty(seg), str(seg))
        
    def testsegAfterCommon1(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [4, 4, 4, 4, -1, -1]
        seg = Alignment.segAfterCommon(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, seg2)
        
    def testsegAfterCommon2(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [3, 3, 4, 4, -1, -1]
        seg = Alignment.segAfterCommon(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [4, 4, 4, 4, -1, -1])
        
    def testsegAfterCommon3(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [3, 3, 3, 3, -1, -1]
        seg = Alignment.segAfterCommon(seg1, seg2,  SEG_START1, SEG_END1)
        self.assert_(Alignment.segEmpty(seg))
        
    def testsegAfterCommon4(self):
        seg1 = [1, 11, 3, 13, -1, -1]
        seg2 = [3, 13, 4, 14, -1, -1]
        seg = Alignment.segAfterCommon(seg1, seg2,  SEG_START1, SEG_END1)
        self.assertEquals(seg, [4, 14, 4, 14, -1, -1])
        
    def testfragmentSegments1(self):
        seg1 = [10, 5, 20, 15, -1, -1]
        seg2 = [15, 10, 25, 20, -1, -1]
        seg = Alignment._fragmentSegments([ seg1, seg2 ])
        Alignment.sort_segments(seg,  SEG_START1)
        self.assertEquals(seg, [ [ 10, 5, 14, 9, -1, -1], [15, 10, 20, 15, -1, -1], [15, 10, 20, 15, -1, -1], [21, 16, 25, 20, -1, -1] ])
        
    def testfragmentSegments2(self):
        seg1 = [10, 10, 20, 20, -1, -1]
        seg2 = [15, 30, 25, 40, -1, -1]
        seg = Alignment._fragmentSegments([ seg1, seg2 ])
        Alignment.sort_segments(seg,  SEG_START1)
        self.assertEquals(seg, [ [ 10, 10, 14, 14, -1, -1], [15, 15, 20, 20, -1, -1], [15, 30, 20, 35, -1, -1], [21, 36, 25, 40, -1, -1] ])
        
    def testfragmentSegments3(self):
        seg1 = [10, 10, 20, 20, -1, -1]
        seg2 = [15, 18, 25, 28, -1, -1]
        seg = Alignment._fragmentSegments([ seg1, seg2 ])
        Alignment.sort_segments(seg,  SEG_START1)
        self.assertEquals(seg, [ [ 10, 10, 14, 14, -1, -1], [15, 15, 17, 17, -1, -1], [15, 18, 17, 20, -1, -1], [18, 18, 20, 20, -1, -1], [18, 21, 20, 23, -1, -1], [21, 24, 25, 28, -1, -1] ])
        
    def testfragmentSegments4(self):
        seg1 = [10, 110, 20, 120, -1, -1]
        seg2 = [10, 110, 25, 125, -1, -1]
        seg = Alignment._fragmentSegments([ seg1, seg2 ])
        Alignment.sort_segments(seg,  SEG_START1)
        self.assertEquals(seg, [ [10, 110, 20, 120, -1, -1], [10, 110, 20, 120, -1, -1], [21, 121, 25, 125, -1, -1]])
        
    def testfragmentSegments5(self):
        seg1 = [5, 105, 20, 120, -1, -1]
        seg2 = [10, 110, 20, 120, -1, -1]
        seg = Alignment._fragmentSegments([ seg1, seg2 ])
        Alignment.sort_segments(seg,  SEG_START1)
        self.assertEquals(seg, [ [5, 105, 9, 109, -1, -1], [10, 110, 20, 120, -1, -1], [10, 110, 20, 120, -1, -1] ])
        
    def testfragmentSegments6(self):
        seg1 = [10, 10, 20, 20, -1, -1]
        seg2 = [30, 50, 40, 60, -1, -1]
        seg = Alignment._fragmentSegments([ seg1, seg2 ])
        Alignment.sort_segments(seg,  SEG_START1)
        self.assertEquals(seg, [ seg1, seg2 ])
        
    def testfragmentSegments7(self):
        seg1 = [20, 1, 30, 11, -1, -1]
        seg2 = [20, 20, 30, 30, -1, -1]
        seg3 = [35, 25, 45, 35, -1, -1]
        seg = Alignment._fragmentSegments([ seg1, seg2, seg3 ])
        Alignment.sort_segments(seg,  SEG_START1)
        self.assertEquals(seg, [ [20, 1, 24, 5, -1, -1], [20, 20, 24, 24, -1, -1], [25, 6, 30, 11, -1, -1], [25, 25, 30, 30, -1, -1], [35, 25, 40, 30, -1, -1], [41, 31, 45, 35, -1, -1] ] )
        
    def testfragmentSegments9(self):
        seg1 = [10, 10, 14, 14, -1, -1]
        seg2 = [10, 11, 14, 15, -1, -1]
        seg = Alignment._fragmentSegments([ seg1, seg2 ])
        Alignment.sort_segments(seg,  SEG_START1)
        segR = [ [x, x, x, x, -1, -1] for x in xrange(10, 15) ]
        segR.extend([ [x, x+1, x, x+1, -1, -1] for x in xrange(10, 15) ])
        Alignment.sort_segments(segR,  SEG_START1)
        self.assertEquals(seg, segR )
        
    def testsegTransitive1(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [1, 1, 3, 3, -1, -1]
        seg = Alignment.segTransitive(seg1, seg2)
        self.assertEquals(seg, [1, 1, 3, 3, -1, -1])
        
    def testsegTransitive2(self):
        seg1 = [1, 1, 3, 3, -1, -1]
        seg2 = [3, 4, 3, 4, -1, -1]
        seg = Alignment.segTransitive(seg1, seg2)
        self.assertEquals(seg, [3, 4, 3, 4, -1, -1])
        
    def testsegTransitive4(self):
        seg1 = [1, 11, 3, 13, -1, -1]
        seg2 = [10, 24, 11, 25, -1, -1]
        seg = Alignment.segTransitive(seg1, seg2)
        self.assertEquals(seg, [1, 25, 1, 25, -1, -1])
        
    def testsegTransitive5(self):
        seg1 = [1, 11, 3, 13, -1, -1]
        seg2 = [8, 24, 10, 26, -1, -1]
        seg = Alignment.segTransitive(seg1, seg2)
        self.assert_(Alignment.segEmpty(seg))
        
    def testsegTransitive6(self):
        seg1 = [1, 11, 3, 13, -1, -1]
        seg2 = [14, 24, 16, 26, -1, -1]
        seg = Alignment.segTransitive(seg1, seg2)
        self.assert_(Alignment.segEmpty(seg))
        
    def testsegTransitive7(self):
        seg1 = [1, 21, 11, 31, -1, -1]
        seg2 = [25, 44, 27, 46, -1, -1]
        seg = Alignment.segTransitive(seg1, seg2)
        self.assertEquals(seg, [5, 44, 7, 46, -1, -1])
        
    def testremoveOverlapSamePhase1(self):
        seg1 = [1, 1, 3, 3, -1, 1]
        seg2 = [4, 4, 4, 4, -1, 2]
        seg = Alignment.removeOverlapSamePhase([ seg1, seg2 ])
        self.assertEquals(seg, [seg1, seg2])
        
    def testremoveOverlapSamePhase2(self):
        seg1 = [1, 11, 3, 13, -1, 1]
        seg2 = [1, 12, 3, 14, -1, 2]
        seg = Alignment.removeOverlapSamePhase([ seg1, seg2 ])
        Alignment.sort_segments(seg,  SEG_START2)
        self.assertEquals(seg, [seg1, seg2])
        
    def testremoveOverlapSamePhase3(self):
        seg1 = [1, 11, 3, 13, -1, 1]
        seg2 = [2, 12, 4, 14, -1, 2]
        seg3 = [3, 13, 5, 15, -1, 4]
        seg = Alignment.removeOverlapSamePhase([ seg1, seg2, seg3 ])
        Alignment.sort_segments(seg,  SEG_START2)
        self.assertEquals(seg, [ [1, 11, 1, 11, -1, 1], [2, 12, 2, 12, -1, 3], [3, 13, 3, 13, -1, 7], [4, 14, 4, 14, -1, 6] , [5, 15, 5, 15, -1, 4] ])
        
    def testremoveOverlapSamePhase4(self):
        seg1 = [1, 11, 3, 13, -1, 1]
        seg2 = [2, 12, 4, 14, -1, 2]
        seg3 = [3, 13, 5, 15, -1, 4]
        seg4 = [4, 14, 6, 16, -1, 8]
        seg = Alignment.removeOverlapSamePhase([ seg1, seg2, seg3, seg4 ])
        Alignment.sort_segments(seg,  SEG_START2)
        self.assertEquals(seg, [ [1, 11, 1, 11, -1, 1], [2, 12, 2, 12, -1, 3], [3, 13, 3, 13, -1, 7], [4, 14, 4, 14, -1, 14] , [5, 15, 5, 15, -1, 12], [6, 16, 6, 16, -1, 8] ])
        
    def testremoveOverlapSamePhase5(self):
        seg1 = [1, 11, 4, 14, -1, 1]
        seg2 = [2, 12, 5, 15, -1, 2]
        seg3 = [3, 13, 6, 16, -1, 4]
        seg4 = [4, 14, 7, 17, -1, 8]
        seg = Alignment.removeOverlapSamePhase([ seg1, seg2, seg3, seg4 ])
        Alignment.sort_segments(seg,  SEG_START2)
        self.assertEquals(seg, [ [1, 11, 1, 11, -1, 1], [2, 12, 2, 12, -1, 3], [3, 13, 3, 13, -1, 7], [4, 14, 4, 14, -1, 15] , [5, 15, 5, 15, -1, 14], [6, 16, 6, 16, -1, 12], [7, 17, 7, 17, -1, 8] ])
        
    def testsegBreak1(self):
        seg1 = [4, 14, 6, 16, -1, 12]
        seg2 = [5, 15, 5, 15, -1, 2]
        seg = []
        Alignment.segBreak(seg1, seg2, seg)
        Alignment.sort_segments(seg,  SEG_START1)
        self.assertEquals(seg, [ [4, 14, 4, 14, -1, 12], [5, 15, 5, 15, -1, 14], [6, 16, 6, 16, -1, 12] ])

    def testis_sorted_sublist1(self):
        self.assert_(Alignment.is_sorted_sublist([], []))

    def testis_sorted_sublist2(self):
        self.assert_(Alignment.is_sorted_sublist([1, 2], []))

    def testis_sorted_sublist3(self):
        self.assert_(Alignment.is_sorted_sublist([1, 2], [1]))

    def testis_sorted_sublist4(self):
        self.assert_(Alignment.is_sorted_sublist([1, 2], [1, 2]))

    def testis_sorted_sublist5(self):
        self.assert_(not Alignment.is_sorted_sublist([], [1]))

    def testis_sorted_sublist6(self):
        self.assert_(not Alignment.is_sorted_sublist([1], [2]))

    def testis_sorted_sublist7(self):
        self.assert_(not Alignment.is_sorted_sublist([1, 2], [0]))

    def testis_sorted_sublist8(self):
        self.assert_(not Alignment.is_sorted_sublist([1, 2], [1, 3]))

    def sort_segments_order1(self):
        seg = [ [1], [2] ]
        order = Alignment.sort_segments_order(seg,  0)
        self.assertEqual(order, [0, 1] )

    def sort_segments_order2(self):
        seg = [ [3], [2] ]
        order = Alignment.sort_segments_order(seg,  0)
        self.assertEqual(order, [1, 0] )

    def sort_segments_order3(self):
        seg = []
        order = Alignment.sort_segments_order(seg,  0)
        self.assertEqual(order, [] )

    def calculateOrderm1_1(self):
        orderm1 = Alignment.calculateOrderm1([])
        self.assertEqual(orderm1, [] )

    def calculateOrderm1_2(self):
        orderm1 = Alignment.calculateOrderm1([1, 0])
        self.assertEqual(orderm1, [1, 0] )

    def calculateOrderm1_3(self):
        orderm1 = Alignment.calculateOrderm1([0, 1])
        self.assertEqual(orderm1, [0, 1] )

    def calculateOrderm1_4(self):
        orderm1 = Alignment.calculateOrderm1([1, 2, 0])
        self.assertEqual(orderm1, [2, 0, 1] )

    def testsegmentNeighbors1(self):
        seg1 = [1, 1, 2, 2, -1, 0]
        seg2 = [3, 3, 4, 4, -1, 0]
        n = Alignment.segmentNeighbors([ seg1, seg2 ], SEG_START1, SEG_END1, [0, 1] )
        self.assertEquals(n, [ [0], [1] ])
        
    def testsegmentNeighbors2(self):
        seg1 = [1, 1, 2, 2, -1, 0]
        seg2 = [2, 2, 3, 3, -1, 0]
        n = Alignment.segmentNeighbors([ seg1, seg2 ], SEG_START1, SEG_END1, [0, 1] )
        self.assertEquals(n, [ [0, 1], [1, 0] ])
        
    def testsegmentNeighbors3(self):
        seg1 = [1, 1, 2, 2, -1, 0]
        seg2 = [2, 2, 3, 3, -1, 0]
        n = Alignment.segmentNeighbors([ seg2, seg1 ], SEG_START1, SEG_END1, [1, 0] )
        self.assertEquals(n, [ [0, 1], [1, 0] ])
        
    def testsegmentNeighbors4(self):
        seg1 = [1, 1, 2, 2, -1, 0]
        seg2 = [2, 2, 3, 3, -1, 0]
        seg3 = [3, 3, 4, 4, -1, 0]
        seg4 = [4, 4, 5, 5, -1, 0]
        n = Alignment.segmentNeighbors([ seg1, seg2, seg3, seg4 ], SEG_START1, SEG_END1, [0, 1, 2, 3] )
        n.sort()
        for s in n: s.sort()
        t = [ [0, 1], [0, 1, 2], [1, 2, 3], [2, 3] ]
        t.sort()
        for s in t: s.sort()
        self.assertEquals(n, t )
        
    def testrealign1(self):
        seg1 = [1102923, 1310121, 1102936, 1310134, -1, 22440]
        seg2 = [1102899, 1310535, 1102994, 1310630, 81, 117324] 
        align = Alignment("s", "f", 1, 2222222, "s2", "f2", 1, 2222222, [seg1, seg2], True)
        align.realign()
        self.assertEqual(align.segments, [[1102899, 1310535, 1102922, 1310558, 81, 117324], [1102923, 1310121, 1102936, 1310134, -1, 22440], [1102937, 1310573, 1102994, 1310630, 81, 117324]])

    def testsegBreakWith1(self):
        seg = [1, 2, 4, 5, 0, 0]
        segr = Alignment.segBreakWith(seg, [])
        self.assertEquals([seg], segr )

    def testsegBreakWith2(self):
        seg = [1, 2, 4, 5, 0, 0]
        segr = Alignment.segBreakWith(seg, [1])
        self.assertEquals(segr, [ [1, 2, 1, 2, 0, 0], [2, 3, 4, 5, 0, 0] ])

    def testsegBreakWith3(self):
        seg = [1, 2, 4, 5, 0, 0]
        segr = Alignment.segBreakWith(seg, [1, 3])
        self.assertEquals(segr, [ [1, 2, 1, 2, 0, 0], [2, 3, 3, 4, 0, 0], [4, 5, 4, 5, 0, 0] ])

    def testcountResidues1(self):
        seg = [ [1, 2, 1, 2, 1, 1] ]
        count = Alignment._countResidues(seg, 1)
        self.assertEquals(count, 1)

    def testcountResidues2(self):
        seg = [ [1, 2, 1, 2, 1, 1] ]
        count = Alignment._countResidues(seg, 2)
        self.assertEquals(count, 0)

    def test_regionsCover1(self):
        seg = [ [1, 2, 2, 3, 1, 1] ]
        r = [ [0, 1], [3, 4] ]
        count = Alignment._regionsCover(r, seg, SEG_START1, SEG_END1)
        self.assertEquals(count, 1)
        self.assertEquals(Alignment.regionsLength(r), 4)

    def test_regionsCover2(self):
        seg = [ [1, 2, 2, 3, 1, 1] ]
        r = [ [0, 1], [3, 4] ]
        count = Alignment._regionsCover(r, seg, SEG_START2, SEG_END2)
        self.assertEquals(count, 1)

    def test_regionsCover3(self):
        seg = [ [1, 2, 2, 3, 1, 1], [1, 2, 3, 4, 1, 1] ]
        r = [ [0, 1], [3, 4] ]
        count = Alignment._regionsCover(r, seg, SEG_START1, SEG_END1)
        self.assertEquals(count, 2)

    def test_regionsMerge1(self):
        r = [ [0, 1], [3, 4] ]
        rr = Alignment._regionsMerge(r[:])
        self.assertEquals(r, rr)

    def test_regionsMerge2(self):
        r = [ [0, 1], [2, 4] ]
        rr = Alignment._regionsMerge(r[:])
        self.assertEquals(r, rr)

    def test_regionsMerge3(self):
        r = [ [0, 1], [1, 4] ]
        rr = Alignment._regionsMerge(r[:])
        self.assertEquals(rr, [ [0, 4] ])

    def test_regionsMerge4(self):
        r = [ [0, 1], [1, 4], [4, 6] ]
        rr = Alignment._regionsMerge(r[:])
        self.assertEquals(rr, [ [0, 6] ])

    def test_regionsMerge5(self):
        r = [ [0, 3], [1, 2] ]
        rr = Alignment._regionsMerge(r[:])
        self.assertEquals(rr, [ [0, 3] ])

    def test_regionsStickyMerge1(self):
        r = [ [0, 1], [3, 4] ]
        rr = Alignment._regionsStickyMerge(r[:])
        self.assertEquals(r, rr)

    def test_regionsStickyMerge2(self):
        r = [ [0, 1], [2, 4] ]
        rr = Alignment._regionsStickyMerge(r[:])
        self.assertEquals([[0,4]], rr)

    def test_regionsStickyMerge3(self):
        r = [ [0, 1], [1, 4] ]
        rr = Alignment._regionsStickyMerge(r[:])
        self.assertEquals(rr, [ [0, 4] ])

    def test_regionsStickyMerge3_1(self):
        r = [ [0, 1, 1], [1, 4, 2] ]
        rr = Alignment._regionsStickyMerge(r[:])
        self.assertEquals(rr, r)

    def test_regionsStickyMerge4(self):
        r = [ [0, 1], [1, 4], [4, 6] ]
        rr = Alignment._regionsStickyMerge(r[:])
        self.assertEquals(rr, [ [0, 6] ])

    def test_regionsStickyMerge4_1(self):
        r = [ [0, 1], [1, 4, 2], [4, 6, 2] ]
        rr = Alignment._regionsStickyMerge(r[:])
        self.assertEquals(rr, [ [0, 1], [1, 6, 2] ])

    def test_regionsStickyMerge5(self):
        r = [ [0, 3], [1, 2] ]
        rr = Alignment._regionsStickyMerge(r[:])
        self.assertEquals(rr, [ [0, 3] ])

    def test_regionsFilter1(self):
        r = [ [0, 0] ]
        seg = [ [1, 1, 3, 3, 1, 1], [1, 2, 2, 3, 1, 1] ]
        s = Alignment._regionsFilter(r[:], seg, SEG_START1, SEG_END1)
        self.assertEquals(s, [ ])

    def test_regionsFilter2(self):
        r = [ [0, 1] ]
        seg = [ [1, 1, 3, 3, 1, 1], [1, 2, 2, 3, 1, 1] ]
        s = Alignment._regionsFilter(r[:], seg, SEG_START1, SEG_END1)
        s.sort()
        self.assertEquals(s, [ [1, 1, 1, 1, 1, 1], [1, 2, 1, 2, 1, 1] ])

    def test_regionsFilter3(self):
        r = [ [4, 4] ]
        seg = [ [1, 1, 3, 3, 1, 1], [5, 7, 6, 8, 1, 1] ]
        s = Alignment._regionsFilter(r[:], seg, SEG_START1, SEG_END1)
        self.assertEquals(s, [ ])

    def test_regionsFilter4(self):
        r = [ [3, 5] ]
        seg = [ [1, 1, 3, 3, 1, 1], [5, 3, 6, 4, 1, 1] ]
        s = Alignment._regionsFilter(r[:], seg, SEG_START1, SEG_END1)
        s.sort()
        self.assertEquals(s, [ [3, 3, 3, 3, 1, 1], [5, 3, 5, 3, 1, 1] ])

    def test_regionsFilter5(self):
        r = [ [3, 5] ]
        seg = [ [1, 1, 3, 3, 1, 1], [1, 2, 3, 4, 1, 1] ]
        s = Alignment._regionsFilter(r[:], seg, SEG_START1, SEG_END1)
        s.sort()
        self.assertEquals(s, [ [3, 3, 3, 3, 1, 1], [3, 4, 3, 4, 1, 1] ])

    def test_resolveOverlap1(self):
        seg = [ ]
        Alignment._resolveOverlap(seg, SEG_START1)

    def test_resolveOverlap2(self):
        seg = [ [1, 1, 3, 3, 1, 1], [1, 2, 2, 3, 1, 1] ]
        Alignment._resolveOverlap(seg, SEG_START1)

    def test_resolveOverlap3(self):
        seg = [ [1, 1, 3, 3, 1, 1], [4, 4, 5, 5, 1, 1] ]
        Alignment._resolveOverlap(seg, SEG_START1)

    def test_resolveOverlap4(self):
        seg = [ [1, 1, 3, 3, 1, 1], [1, 1, 3, 3, 1, 1] ]
        self.assertRaises(AssertionError, Alignment._resolveOverlap, seg, SEG_START1)

    def test_resolveOverlap5(self):
        seg = [ [1, 1, 3, 3, 1, 1], [3, 3, 3, 3, 1, 1] ]
        self.assertRaises(AssertionError, Alignment._resolveOverlap, seg, SEG_START1)

    def test_resolveOverlap6(self):
        seg = [ [1, 1, 3, 3, 1, 1], [2, 3, 2, 3, 1, 1], [3, 3, 3, 3, 1, 1] ]
        self.assertRaises(AssertionError, Alignment._resolveOverlap, seg, SEG_START1)

    def test_resolveOverlap7(self):
        seg = [ [1, 1, 1, 1, 1, 1], [2, 3, 2, 3, 1, 1], [1, 1, 3, 3, 1, 1] ]
        self.assertRaises(AssertionError, Alignment._resolveOverlap, seg, SEG_START1)
        
    def test__segmentsDiff1(self):
        seg1 = [ [1, 1, 1, 1, 1, 1] ]
        seg2 = [ [1, 1, 1, 1, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [])

    def test__segmentsDiff2(self):
        seg1 = [ [1, 2, 1, 2, 1, 1] ]
        seg2 = [ [1, 1, 1, 1, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [1, 2, 1, 2, 1, 1] ])

    def test__segmentsDiff3(self):
        seg1 = [ [1, 1, 1, 1, 1, 1], [1, 2, 1, 2, 1, 1] ]
        seg2 = [ [1, 1, 1, 1, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [1, 2, 1, 2, 1, 1] ])

    def test__segmentsDiff4(self):
        seg1 = [ [1, 0, 1, 0, 1, 1], [1, 1, 1, 1, 1, 1]  ]
        seg2 = [ [1, 1, 1, 1, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [1, 0, 1, 0, 1, 1] ])

    def test__segmentsDiff5(self):
        seg1 = [ [105, 110, 115, 120, 1, 1] ]
        seg2 = [ [110, 110, 120, 120, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [110, 115, 115, 120, 1, 1] ])

    def test__segmentsDiff6(self):
        seg1 = [ [110, 110, 120, 120, 1, 1], [105, 110, 115, 120, 1, 1] ]
        seg2 = [ [110, 110, 120, 120, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [110, 115, 115, 120, 1, 1] ])

    def test__segmentsDiff7(self):
        seg1 = [ [115, 110, 125, 120, 1, 1] ]
        seg2 = [ [110, 110, 120, 120, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [115, 110, 120, 115, 1, 1] ])

    def test__segmentsDiff8(self):
        seg1 = [ [115, 110, 125, 120, 1, 1] ]
        seg2 = [ [110, 110, 120, 120, 1, 1], [130, 130, 140, 140, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [115, 110, 120, 115, 1, 1] ])

    def test__segmentsDiff9(self):
        seg1 = [ [115, 110, 125, 120, 1, 1] ]
        seg2 = [ [100, 100, 109, 109, 1, 1], [110, 110, 120, 120, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [115, 110, 120, 115, 1, 1] ])

    def test__segmentsDiff10(self):
        seg1 = [ [105, 110, 115, 120, 1, 1] ]
        seg2 = [ [110, 110, 120, 120, 1, 1], [115, 110, 125, 120, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [110, 115, 115, 120, 1, 1] ])

    def test__segmentsDiff11(self):
        seg1 = [ [105, 110, 115, 120, 1, 1], [120, 120, 130, 130, 1, 1] ]
        seg2 = [ [110, 170, 120, 180, 1, 1], [115, 115, 125, 125, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [110, 115, 115, 120, 1, 1], [120, 120, 120, 120, 1, 1] ])

    def test__segmentsDiff12(self):
        seg1 = [ [110, 210, 150, 250, 1, 1] ]
        seg2 = [ [105, 105, 110, 110, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [110, 210, 110, 210, 1, 1] ])

    def test__segmentsDiff13(self):
        seg1 = [ [110, 210, 150, 250, 1, 1] ]
        seg2 = [ [105, 105, 115, 115, 1, 1], [116, 116, 120, 120, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [110, 210, 120, 220, 1, 1] ])

    def test__segmentsDiff14(self):
        seg1 = [ [110, 210, 150, 250, 1, 1] ]
        seg2 = [ [105, 105, 115, 115, 1, 1], [117, 117, 120, 120, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [110, 210, 115, 215, 1, 1], [117, 217, 120, 220, 1, 1] ])

    def test__segmentsDiff15(self):
        seg1 = [ [110, 210, 150, 250, 1, 1] ]
        seg2 = [ [105, 105, 115, 115, 1, 1], [117, 117, 120, 120, 1, 1], [140, 140, 160, 160, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [110, 210, 115, 215, 1, 1], [117, 217, 120, 220, 1, 1], [140, 240, 150, 250, 1, 1] ])

    def test__segmentsDiff16(self):
        seg1 = [ [110, 210, 150, 250, 1, 1] ]
        seg2 = [ [105, 105, 115, 115, 1, 1], [117, 117, 120, 120, 1, 1], [125, 125, 135, 135, 1, 1], [140, 140, 160, 160, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [110, 210, 115, 215, 1, 1], [117, 217, 120, 220, 1, 1], [125, 225, 135, 235, 1, 1], [140, 240, 150, 250, 1, 1] ])

    def test__segmentsDiff17(self):
        seg1 = [ [110, 210, 150, 250, 1, 1], [120, 220, 150, 250, 1, 1] ]
        seg2 = [ [105, 105, 130, 130, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [110, 210, 130, 230, 1, 1], [120, 220, 130, 230, 1, 1] ])

    def test__segmentsDiff18(self):
        seg1 = [ [110, 210, 150, 250, 1, 1] ]
        seg2 = [ [105, 105, 115, 115, 1, 1], [115, 115, 120, 120, 1, 1], [125, 125, 135, 135, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [110, 210, 120, 220, 1, 1], [125, 225, 135, 235, 1, 1] ])

    def test__segmentsDiff19(self):
        seg1 = [ [110, 210, 150, 250, 1, 1], [170, 270, 190, 290, 1, 1] ]
        seg2 = [ [130, 130, 180, 180, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [130, 230, 150, 250, 1, 1], [170, 270, 180, 280, 1, 1] ])

    def test__segmentsDiff20(self):
        seg1 = [ [110, 210, 150, 250, 1, 1] ]
        seg2 = [ [120, 120, 130, 130, 1, 1], [130, 130, 140, 140, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [120, 220, 140, 240, 1, 1] ])

    def test__segmentsDiff21(self):
        seg1 = [ [110, 210, 150, 250, 1, 1] ]
        seg2 = [ [120, 120, 130, 130, 1, 1], [131, 131, 140, 140, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [120, 220, 140, 240, 1, 1] ])

    def test__segmentsDiff22(self):
        seg1 = [ [110, 210, 150, 250, 1, 1], [130, 231, 140, 241, 1, 1] ]
        seg2 = [ [120, 120, 125, 125, 1, 1], [131, 131, 131, 131, 1, 1], [141, 141, 155, 155, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        seg3.sort()
        self.assertEquals(seg3, [ [120, 220, 125, 225, 1, 1], [131, 231, 131, 231, 1, 1], [131, 232, 131, 232, 1, 1], [141, 241, 150, 250, 1, 1] ] )

    def test__segmentsDiff23(self):
        seg1 = [ [110, 210, 150, 250, 1, 1], [130, 231, 140, 241, 1, 1] ]
        seg2 = [ [120, 120, 125, 125, 1, 1], [141, 141, 155, 155, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        self.assertEquals(seg3, [ [120, 220, 125, 225, 1, 1], [141, 241, 150, 250, 1, 1] ] )

    def test__segmentsDiff24(self):
        seg1 = [ [110, 210, 150, 250, 1, 1], [130, 231, 140, 241, 1, 1] ]
        seg2 = [ [120, 120, 125, 125, 1, 1], [131, 131, 131, 131, 1, 1], [141, 141, 155, 155, 1, 1] ]
        seg3 = Alignment._segmentsDiff(seg1, seg2, SEG_START1, SEG_END1)
        seg3.sort()
        self.assertEquals(seg3, [ [120, 220, 125, 225, 1, 1], [131, 231, 131, 231, 1, 1], [131, 232, 131, 232, 1, 1], [141, 241, 150, 250, 1, 1] ] )

if __name__ == '__main__':
    unittest.main()